/***************************************************************************
 *     Copyright 2001  Broadcom Corporation
 *     All Rights Reserved
 *     No portions of this material may be reproduced in any form without the
 *     written permission of:
 *              Broadcom Corporation
 *              16251 Laguna Canyon Road
 *              Irvine, California  92618
 *     All information contained in this document is Broadcom Corporation
 *     company private, proprietary, and trade secret.
 *
 ****************************************************************************
 *
 *     Filename:  rekeyReplay.c
 *     Creation Date: 5 November 2001 (v0.00)
 *     VSS Info:
 *         $Revision: 1.1 $
 *         $Date: 2002/08/21 17:54:10 $
 *
 ****************************************************************************
 *     Description:
 *
 *       Implementation of rekey replay
 *
 *     Notes:
 *
 ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "rekeyReplay.h"
#include "seckrb.h"

#ifdef DEMO
#define GLOBAL_CFG_MAX_ENDPT   8
#else
#include <globalCfg.h>
#endif

/* 4 rekey replays per endpoint */
#define MAX_REKEY_REPLAY_ENTRIES    4*GLOBAL_CFG_MAX_ENDPT

/* structs using in table to map info to NVRAM */
typedef struct
{
   unsigned char   t_hash[MD5_SIZE];          /* hash of cms fqdn */
   unsigned long   time_stamp;
   unsigned long   nonce;
} rekeyReplay_Info;


static rekeyReplay_Info rekeyReplay_Table[MAX_REKEY_REPLAY_ENTRIES];



/*
******************************************************************************
** FUNCTION:  secStoreRekeyMessage
**
** PURPOSE:   store a rekey message, (only nonce, fqdn, timestamp)
**
** PARAMETERS:
**    1. nonce
**    2. fqdn of call agent
**    3. timestamp form the rekey message
**
**
** RETURNS:
**
** NOTE:
**
******************************************************************************
*/
void secStoreRekeyMessage(
      unsigned long   nonce,
      unsigned char   *fqdn,
      unsigned long   time_stamp)
{
   int       i;
   int       tmp_len;
   time_t    curtime;

   /* First cleanup any expired entries */
   for (i=0;i<MAX_REKEY_REPLAY_ENTRIES;i++)
   {
      time(&curtime);
      if ( (labs(curtime - rekeyReplay_Table[i].time_stamp)) >
            MAX_ALLOWED_CLOCK_SKEW_IN_SECONDS)
      {
         rekeyReplay_Table[i].nonce = 0;
         rekeyReplay_Table[i].time_stamp = 0;
         rekeyReplay_Table[i].t_hash[0] = '\0';
      }
   }

   for (i=0;i<MAX_REKEY_REPLAY_ENTRIES;i++)
   {
      if ( ! rekeyReplay_Table[i].nonce )
      {
         rekeyReplay_Table[i].nonce = nonce;
         rekeyReplay_Table[i].time_stamp = time_stamp;
         SecLibGetCryptoIface()->secCryptoCalcHash(HASH_MD5_TYPE, fqdn, strlen(fqdn),
               NULL, 0, (unsigned char *)(rekeyReplay_Table[i].t_hash),
               &tmp_len);
         return;
      }
   }

   if ( i == MAX_REKEY_REPLAY_ENTRIES )
      printf("SEC PANIC: NO MORE REKEY REPLAY TABLE ENTRIES LEFT.");
}



/*
******************************************************************************
** FUNCTION:  secCheckRekeyReplay
**
** PURPOSE:   Check if a rekey message is a replay
**
** PARAMETERS:
**    1. nonce
**    2. fqdn of call agent
**    3. timestamp form the rekey message
**
**
** RETURNS:
**    FOUND or NOT_FOUND
**
** NOTE:
**
******************************************************************************
*/
int secCheckRekeyReplay(
      unsigned long   nonce,
      unsigned char   *fqdn,
      unsigned long   time_stamp)
{
   int           i;
   unsigned char tmp_hash[MD5_SIZE];
   int           tmp_len;
   time_t        curtime;

   for (i=0;i<MAX_REKEY_REPLAY_ENTRIES;i++)
   {
      if ( nonce == rekeyReplay_Table[i].nonce )
      {
         /* If nonces match, see if rekey is from same call agent */
         SecLibGetCryptoIface()->secCryptoCalcHash(HASH_MD5_TYPE, fqdn, strlen(fqdn),
               NULL, 0, tmp_hash, &tmp_len);

         if ( ! memcmp(rekeyReplay_Table[i].t_hash, tmp_hash, MD5_SIZE))
         {
            /* see if this rekey has been within the time skew */
            time(&curtime);
            if ( (labs(curtime - time_stamp)) > MAX_ALLOWED_CLOCK_SKEW_IN_SECONDS)
            {
               printf("Rekey Replay stored has expired, removing.");
               rekeyReplay_Table[i].nonce = 0;
               rekeyReplay_Table[i].time_stamp = 0;
               rekeyReplay_Table[i].t_hash[0] = '\0';
            }
            else
            {
               printf("This rekey is a replay.");
               return REKEY_REPLAY_FOUND;
            }
         }
      }
   }
   return REKEY_REPLAY_NOT_FOUND;
}
