/***************************************************************************
 *     Copyright 2004  Broadcom Corporation
 *     All Rights Reserved
 *     No portions of this material may be reproduced in any form without the
 *     written permission of:
 *              Broadcom Corporation
 *              16251 Laguna Canyon Road
 *              Irvine, California  92618
 *     All information contained in this document is Broadcom Corporation
 *     company private, proprietary, and trade secret.
 *
 ****************************************************************************
 *
 *     Filename:  SecNVRAM_ch.c
 *     Creation Date: 11-12-2004
 *     
 ****************************************************************************
 *     Description:
 *
 *       Implementation of wrappers to inerface to NVRAM for security
 *
 *     Notes:
 *
 ****************************************************************************/


#include <stdio.h>
#include "SecLib.h"

#include <SecUtils.h>


/* structs using in table to map info to NVRAM */
typedef struct
{
    unsigned char   t_hash[MD5_SIZE];          /* hash of realm */
    unsigned char   flag;                /* prov or cms */
    int             skew;                /* time skew value */
} nvram_skew_info;


#if (0)
typedef struct 
{
    unsigned char   t_hash[MD5_SIZE];          /* hash of realm */
    int             index;               
} nvram_kdc_index_info;

typedef struct 
{
    unsigned char   t_hash[MD5_SIZE];          /* hash of fqdn */
    int             index;                
} nvram_cms_index_info;
#endif

typedef struct
{
    int              cert_len;
    unsigned char    cert[MAX_CERTIFICATE_LENGTH];
} nvram_certificate;

typedef struct
{
    int              key_len;
    unsigned char    key[MAX_CERT_KEY_LENGTH];
} nvram_key;



/* tables to map information to VNRAM */
/* for time skew, max entries is 2 for provisioning (AS or AP) (2)
 * and 1 per endpoint per AS or AP for cms enrty (4*2) totals 10
 * for tkk, session key we need half of that.
 * Tkt, session key, 1 for prov and maximum of 4 per endpoints
 * for CMS totals 5
 * */
#define MAX_NVRAM_TABLE_ENTRIES         8


//static   unsigned char tmp_tkt[MAX_TKT_LENGTH];
//static   unsigned char tmp_skey[MAX_SESSION_KEY_LENGTH];
//static   unsigned char tmp_fqdn[255];

#ifdef DEMO

/* certificates and keys */
static nvram_certificate         MTA_man_cert;
static nvram_certificate         MTA_dev_cert;
static nvram_key                 MTA_device_modulus_key;
static nvram_key                 MTA_device_priv_key;
static nvram_certificate         TEL_root_cert;

#define MAX_NVRAM_SKEW_TABLE_ENTRIES    10
static nvram_skew_info           skew_table[MAX_NVRAM_SKEW_TABLE_ENTRIES];
#endif


/*
******************************************************************************
** FUNCTION:  secWriteNVRAMSkew
**
** PURPOSE:   Write time skew into NVRAM
**
** PARAMETERS:
**    1. time skew value
**    2. flag prov or cms (AS or AP)
**    3. realm to be hashed
**    4. length of realm
**
**
** RETURNS:
**
** NOTE:
**
******************************************************************************
*/
void ch_secWriteNVRAMSkew(
                         int             skew_value,
                         int             flag,
                         char            *realm,
                         unsigned long   realm_length)
{
    int           i;
    unsigned long tmp_len;

    /* find an open entry in the table */
    for ( i=0;i<MAX_NVRAM_SKEW_TABLE_ENTRIES;i++ )
        if ( skew_table[i].t_hash[0] == '\0' )
            break;
    if ( i == MAX_NVRAM_SKEW_TABLE_ENTRIES )
        printf("*SEC PANIC* no more NVRAM Skew_table entries.\n");

    SecLibGetCryptoIface()->secCryptoCalcHash(HASH_MD5_TYPE, realm, realm_length,
                                              NULL, 0, (unsigned char *)(skew_table[i].t_hash), &tmp_len);

    skew_table[i].flag = (unsigned char)flag;
    skew_table[i].skew = skew_value;
}

/*
******************************************************************************
** FUNCTION:  secReadNVRAMSkew
**
** PURPOSE:   Read time skew into NVRAM
**
** PARAMETERS:
**    1. time skew value
**    2. flag prov or cms (AS or AP)
**    3. realm to be hashed
**    4. length of realm
**    5. if match, index to be used when writing
**
**
** RETURNS:
**
** NOTE:
**
******************************************************************************
*/
int ch_secReadNVRAMSkew(
                       int             flag,
                       char            *realm,
                       unsigned long   realm_length)
{
    unsigned char tmp_hash[MD5_SIZE];
    unsigned long tmp_len;
    int           i;


    SecLibGetCryptoIface()->secCryptoCalcHash(
                                             HASH_MD5_TYPE, realm, realm_length, NULL, 0,
                                             tmp_hash, &tmp_len);

    for ( i=0;i<MAX_NVRAM_SKEW_TABLE_ENTRIES;i++ )
    {
        if ( ! memcmp(skew_table[i].t_hash, tmp_hash, MD5_SIZE) )
        {
            if ( skew_table[i].flag == flag )
            {
                return(skew_table[i].skew);
            }
        }
    }
    return 0;
}

/*
******************************************************************************
** FUNCTION:  secWriteNVRAMTkt
**
** PURPOSE:   Write ticket into NVRAM
**
** PARAMETERS:
**    1. asn.1 encoded ticket
**    2. flag prov or cms (AS or AP)
**    3. cms fqdn, ignored if prov
**    4. length of fqdn
**
**
** RETURNS:
**
** NOTE:
**
******************************************************************************
*/
void ch_secWriteNVRAMTkt(
                        unsigned char   *tkt,
                        int             tkt_len,
                        int             flag,
                        char            *fqdn,
                        unsigned long   fqdn_length)
{
    return;
}

/*
******************************************************************************
** FUNCTION:  secReadNVRAMTkt
**
** PURPOSE:   Read ticket into NVRAM
**
** PARAMETERS:
**    1. asn.1 encoded ticket
**    2. flag prov or cms (AS or AP)
**    3. cms fqdn, ignored if prov
**    4. length of fqdn
**
**
** RETURNS:
**    Length of the ticket
**
** NOTE:
**    Space for the ticket must be allocated before this call
**
******************************************************************************
*/
unsigned long ch_secReadNVRAMTkt(
                                unsigned char   *tkt,
                                unsigned long   tkt_len,
                                int             flag,
                                char            *fqdn,
                                unsigned long   fqdn_length)
{
    return 0;
}


/*
******************************************************************************
** FUNCTION:  secWriteNVRAMSessionKeyAndTktTimes
**
** PURPOSE:   Write session key and ticket times into NVRAM
**
** PARAMETERS:
**    1. session key always 24 bytes
**    2. length of session key
**    3. type of session key
**    4. flag prov or cms (AS or AP)
**    5. cms fqdn, ignored if prov
**    6. length of fqdn
**    7. ticket times (4 integers)
**
** RETURNS:
**
** NOTE:
**
******************************************************************************
*/
void secWriteNVRAMSessionKeyAndTktTimes(
                                       unsigned char   *session_key,
                                       unsigned long   session_key_len,
                                       int             session_key_type,
                                       int             flag,
                                       char            *fqdn,
                                       unsigned long   fqdn_length,
                                       int             auth_time,
                                       int             start_time,
                                       int             end_time,
                                       int             renew_till)
{
    return;
}

/*
******************************************************************************
** FUNCTION:  secReadNVRAMSessionKeyAndTktTimes
**
** PURPOSE:   Read session key and ticket times from NVRAM
**
** PARAMETERS:
**    1. session key
**    2. session key type
**    3. flag prov or cms (AS or AP)
**    4. cms fqdn, ignored if prov
**    5. length of fqdn
**    6. ticket times (4 integers)
**
**
** RETURNS:
**    length of the session key or 0
**
** NOTE:
**    caller must allocate space for session key
**
******************************************************************************
*/
unsigned long ch_secReadNVRAMSessionKeyAndTktTimes(
                                                  unsigned char   *session_key,
                                                  int             *session_key_type,
                                                  int             flag,
                                                  char            *fqdn,
                                                  unsigned long   fqdn_length,
                                                  int             *auth_time,
                                                  int             *start_time,
                                                  int             *end_time,
                                                  int             *renew_till)
{
    return 0;
}


/*
******************************************************************************
** FUNCTION:  secReadNVRAMMTACertKeyData
**
** PURPOSE:   Read manufacture certificate from NVRAM
**
** PARAMETERS:
**    1. in: certificate type
**    2. out: certificate length
**    3. out: certificate
**
**
** RETURNS:
**    Success or failure
**
** NOTE:
**    caller responsible for allocating and freeing memory
******************************************************************************
*/
unsigned long ch_secReadNVRAMMTACertKeyData(
                                           SEC_CERT_TYPE   type,
                                           unsigned int    *len,
                                           unsigned char   *data)
{
    switch ( type )
    {
    case MTA_MAN_CERT:
        *len = MTA_man_cert.cert_len;
        memcpy(data, MTA_man_cert.cert, *len);
        return 1;
    case MTA_DEV_CERT:
        *len = MTA_dev_cert.cert_len;
        memcpy(data, MTA_dev_cert.cert, *len);
        return 1;
    case MAT_DEV_MOD_KEY:
        *len = MTA_device_modulus_key.key_len;
        memcpy(data, MTA_device_modulus_key.key, *len);
        return 1;
    case MTA_DEV_PRIV_KEY:
        *len = MTA_device_priv_key.key_len;
        memcpy(data, MTA_device_priv_key.key, *len);
        return 1;
    case TEL_ROOT_CERT:
        *len = TEL_root_cert.cert_len;
        memcpy(data, TEL_root_cert.cert, *len);
        return 1;
    default:
        printf("NVRAM secReadNVRAMMTACertKeyData bad type %d.\n", type);
        return 0;
    } 
}

/*
******************************************************************************
** FUNCTION: secWriteNVRAMMTACertKeyData
**
** PURPOSE:   write a certificate or a key from NVRAM
**
** PARAMETERS:
**    1. in: certificate or key type
**    2. in: certificate or key length
**    3. in: certificate or key
**
**
** RETURNS:
**    Success or failure
**
** NOTE:
**    caller responsible for allocating and freeing memory
******************************************************************************
*/
unsigned long ch_secWriteNVRAMMTACertKeyData(
                                            SEC_CERT_TYPE   type,
                                            int             len,
                                            const unsigned char   *data)
{
    switch ( type )
    {
    case MTA_MAN_CERT:
        if ( len > MAX_CERTIFICATE_LENGTH )
        {
            printf("NVRAM secWriteNVRAMMTACertKeyData Manufacture cert too big. %d\n", len);
            return 0;
        }
        MTA_man_cert.cert_len = len;
        memcpy(MTA_man_cert.cert, data, len);
        return 1;

    case MTA_DEV_CERT:
        if ( len > MAX_CERTIFICATE_LENGTH )
        {
            printf("NVRAM secWriteNVRAMMTACertKeyData Device cert too big. %d\n", len);
            return 0;
        }
        MTA_dev_cert.cert_len = len;
        memcpy(MTA_dev_cert.cert, data, len);
        return 1;

    case MAT_DEV_MOD_KEY:
        if ( len > MAX_KEY_LENGTH )
        {
            printf("NVRAM secWriteNVRAMMTACertKeyData Device modulus too big. %d\n", len);
            return 0;
        }
        MTA_device_modulus_key.key_len = len;
        memcpy(MTA_device_modulus_key.key, data, len);
        return 1;

    case MTA_DEV_PRIV_KEY:
        if ( len > MAX_KEY_LENGTH )
        {
            printf("NVRAM secWriteNVRAMMTACertKeyData Device private key too big. %d\n", len);
            return 0;
        }
        MTA_device_priv_key.key_len = len;
        memcpy(MTA_device_priv_key.key, data, len);
        return 1;

    case TEL_ROOT_CERT:
        if ( len > MAX_CERTIFICATE_LENGTH )
        {
            printf("NVRAM secWriteNVRAMMTACertKeyData Telephony Root Cert too big. %d\n", len);
            return 0;
        }
        TEL_root_cert.cert_len = len;
        memcpy(TEL_root_cert.cert, data, len);
        return 1;

    default:
        printf("NVRAM secWriteNVRAMMTACertKeyData bad type %d.\n", type);
        return 0;
    }
    return 1;
}
