/***************************************************************************
*    Copyright 2000  Broadcom Corporation
*    All Rights Reserved
*    No portions of this material may be reproduced in any form without the
*    written permission of:
*             Broadcom Corporation
*             16251 Laguna Canyon Road
*             Irvine, California  92618
*    All information contained in this document is Broadcom Corporation
*    company private, proprietary, and trade secret.
*
****************************************************************************
*
*    Filename: bcmos.c
*    Creation Date: 8 June 2000 (v0.00)
*    VSS Info:
*        $Revision: 1.2 $
*        $Date: 2002/12/23 15:13:36 $
*
****************************************************************************
*    Description:
*
*     Broadcom Generic Operating System Functions. This file should only
*     contain functions which common to all operating system types.
*     Operating system specific implementations of the BCMOS API should
*     be contained within the OS-specific sub-folder.
*
****************************************************************************/

//#include <hausenv.h>
//#include <string.h>
//#include <bcmos.h>
#include "osdef.h"

#if CH_NEEDED

/*
*****************************************************************************
** TYPE DEFINITIONS and CONSTANTS
*****************************************************************************
*/


/*
*****************************************************************************
** LOCAL VARIABLES
*****************************************************************************
*/

/* Priority level structure for EMTA threads. */
static OS_THREAD_CLASS_PRIO  osTaskPriorities;

/*****************************************************************************
** FUNCTION:   bcmOsInit
**
** PURPOSE:    initialize any os specific variables
**
** PARAMETERS: priorities - structure that contains the absolute priority
**                          levels associated with each class of MTA task
**
** RETURNS:    OS_STATUS_OK
*****************************************************************************/
OS_STATUS bcmOsInit( OS_THREAD_CLASS_PRIO *priorities )
{
   OS_STATUS               status;

   /* Store task priorities in local structure */
   osTaskPriorities = *priorities;

   /* Initialize the operating system */
   if (( status = osInit()) != OS_STATUS_OK )
   {
      return ( status );
   }

   return( OS_STATUS_OK );
}


/*****************************************************************************
** FUNCTION:   bcmOsTaskPriority
**
** PURPOSE:    To retrieve the absolute task priority based upon the task class
**
** PARAMETERS: taskClass [IN] - the type of MTA class
**             priority [OUT] - priority level
**
** RETURNS:    OS_STATUS
*****************************************************************************/
OS_STATUS bcmOsTaskPriority( OS_TASK_CLASS taskClass, UINT32 *priority )
{
   if ( taskClass >= 0 )
   {
      /* Priority is not one of the pre-defined ennumerated ones... do not translate */
      *priority = taskClass;
   }
   else
   {
      /* Use translation table to convert from enumerated priority class to associated
       * underlying OS priority */
      *priority = osTaskPriorities.priorityLevel[ OS_TASK_TO_INDEX( taskClass ) ];
   }

   return ( OS_STATUS_OK );
}


/*****************************************************************************
** FUNCTION:   bcmOsTaskClass
**
** PURPOSE:    To retrieve the task class based upon the absolute task priority
**
** PARAMETERS: priority [IN]     - OS-specific, absolute task priority
**             taskClass [OUT]   - task class types
**
** RETURNS:    OS_STATUS
*****************************************************************************/
OS_STATUS bcmOsTaskClass( UINT32 priority, OS_TASK_CLASS *taskClass )
{

   /* There is no guarantee that each OS_TASK_CLASS is bound to a unique
    * priority. Therefore, we can't iterate through the list of real OS
    * priorities, find a match, and assume that we have found the correct
    * task class. We will simply return the real priority as the task class.
    * The real OS task priority is useful to the application because it can
    * be used set the task priority when creating a new task (instead of
    * specifying the task class). */
   *taskClass = priority;

   return ( OS_STATUS_OK );
}
#endif
/*****************************************************************************
** FUNCTION:   bcmOsConvertSocketOptions
**
** PURPOSE:    To convert from OS-independent socket options to OS-dependent
**             values.
**
** PARAMETERS: offset      - offset into the OS-dependent array
**
** RETURNS:    OS_STATUS
******************************************************************************/
UINT32 bcmOsConvertSocketOptions(OS_SOCKOPT offset)
{
   /* verify that the option name is valid */
   ASSERT ( 0 <= offset < OS_SOCKET_OPT_MAX);

   /* Since OS-independent values are defined as a set of enums, we can use
      these values as offsets into the OS-independent array */
   return (osSockOpt[offset]);
}


