/***************************************************************************
*    Copyright 2000  Broadcom Corporation
*    All Rights Reserved
*    No portions of this material may be reproduced in any form without the
*    written permission of:
*             Broadcom Corporation
*             16251 Laguna Canyon Road
*             Irvine, California  92618
*    All information contained in this document is Broadcom Corporation
*    company private, proprietary, and trade secret.
*
****************************************************************************
*
*    Filename: socket.c
*    Creation Date: 28 June 2001 (v0.00)
*    VSS Info:
*        $Revision: 1.4 $
*        $Date: 2004/10/04 19:52:52 $
*
****************************************************************************
*    Description:
*
*     Broadcom Generic Operating System Functions. This file contains the VxWorks
*     specific implementation of semaphores.
*
****************************************************************************/

#if CH_NEEDED
#include <hausenv.h>
#include <string.h>
#include <in.h>
#include <bcmos.h>
#endif

#include "socket.h"

/* Operating system specific includes */
#include <vxWorks.h>
#include <sockLib.h>
#include <unistd.h>

/* OS-dependent socket options */
UINT32 osSockOpt[OS_SOCKET_OPT_MAX];

extern int ktracelevel;

/*****************************************************************************
** FUNCTION:   socketInit
**
** PURPOSE:    Initialize any socket related interfaces
**
** PARAMETERS: None
**
** RETURNS:    OS_STATUS_OK
*****************************************************************************/
#if CH_NEEDED
OS_STATUS socketInit( void )
{
    /* initialize OS-dependent socket options */
    osSockOpt[OS_AF_INET] = AF_INET;
    osSockOpt[OS_SOCK_DGRAM] = SOCK_DGRAM;
    osSockOpt[OS_SOL_SOCKET] = SOL_SOCKET;
    osSockOpt[OS_SO_REUSEADDR] = SO_REUSEADDR;
    osSockOpt[OS_INADDR_ANY] = INADDR_ANY;
    osSockOpt[OS_IPPROTO_IP] = IPPROTO_IP;
    osSockOpt[OS_IPPROTO_UDP] = IPPROTO_UDP;
    osSockOpt[OS_IP_TOS] = IP_TOS;

    return( OS_STATUS_OK );

}
#endif

/*****************************************************************************
** FUNCTION:   bcmOsSocketShare
**
** PURPOSE:    allows a task access to a socket.
**
** PARAMETERS: socketDesc    - Socket Descriptor
**             taskID        - task identifier
**             newSocketDesc - task's socket descriptor (output)
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
**
******************************************************************************/
#if CH_NEEDED
OS_STATUS bcmOsSocketShare( int socketDesc, OS_TASKID taskID, int *newSocketDesc)
{

    ASSERT( newSocketDesc );

    *newSocketDesc = socketDesc;

    /* Error checking */
    return( checkSocketStatus((NATIVE_SOCKET_STATUS) *newSocketDesc, "bcmOsSocketShare" ));
}
#endif

/*****************************************************************************
** FUNCTION:   bcmOsSocketCreate
**
** PURPOSE:    creates a new socket.
**
** PARAMETERS:
**             domain        - socket domain
**             type          - socket type carrier
**             protocol      - socket protocol type
**             descriptor    - socket descriptor
**
** RETURNS:    OS_STATUS_OK  - success
**             error code    - failure
**
** NOTE:
**
******************************************************************************/
#if CH_NEEDED
OS_STATUS bcmOsSocketCreate (OS_SOCKOPT domain, OS_SOCKOPT type, int protocol, int *descriptor)
{
    UINT32 osDomain;
    UINT32 osType;

    ASSERT( descriptor );

    /* convert form OS-independent values to OS-dependent values */
    osDomain = bcmOsConvertSocketOptions(domain);
    osType = bcmOsConvertSocketOptions(type);

    *descriptor = (int) socket( osDomain, osType, protocol );

    /* Error checking */
    return( checkSocketStatus((NATIVE_SOCKET_STATUS) *descriptor, "bcmOsSocketCreate" ));
    /* int NewSock;
     if ((NewSock = socket(AF_INET, SOCK_DGRAM, 0)) == -1)
     { 
         printf("bcmOsSocketCreate:  COULD NOT CREATE SOCKET\n");
         return -1; //error
     }
     *descriptor = NewSock;
     printf("bcmOsSocketCreate:  created socket with descriptor <%d>\n",NewSock);
 
     return 0;
     */
}

#endif
/*****************************************************************************
** FUNCTION:   bcmOsSocketBind
**
** PURPOSE:    Binds a name/address to a socket
**
** PARAMETERS:
**             descriptor    - socket descriptor
**             name          - address/name structure to be bound
**             namelen       - size of structure
**
** RETURNS:    OS_STATUS_OK  - success
**             error code    - failure
**
** NOTE:
**
******************************************************************************/
#if CH_NEEDED
OS_STATUS bcmOsSocketBind ( int descriptor, OS_SOCKADDR *name, int namelen)
{
    NATIVE_SOCKET_STATUS    status;
    OS_SOCKADDR            *osName;

    ASSERT( name );

    /* update OS-independent parameters to OS-dependent values */
    osName = name;
    if ( name->sin_family == OS_AF_INET )
    {
        osName->sin_family = (short)bcmOsConvertSocketOptions((UINT32)name->sin_family);
    }
    if ( name->sin_addr.S_addr == OS_INADDR_ANY )
    {
        osName->sin_addr.S_addr = (short)bcmOsConvertSocketOptions((UINT32)name->sin_addr.S_addr);
    }

    status = bind( descriptor, (struct sockaddr_in*) osName, namelen );

    /* Error checking */
    return( checkSocketStatus( status, "bcmOsSocketBind" ));
}
#endif
/*****************************************************************************
** FUNCTION:   bcmOsSocketReceiveFrom
**
** PURPOSE:    receive a message from a socket
**
** PARAMETERS:
**             socketDesc    - socket to receive from
**             bufp          - pointer to data buffer
**             buflen        - length of buffer
**             flags         - flags to underlying protocols
**             ifno           - specifies outgoing interface for message
**                               (0 for default interface)
**             fromp         - where to copy sender's address
**             pFromLen      - value/result length of from
**             numBytesRec   - number of bytes received.
**
** RETURNS:    OS_STATUS_OK  - success
**             error code    - failure
**
** NOTE:
**
******************************************************************************/
OS_STATUS bcmOsSocketReceiveFrom ( int socketDesc, char *bufp, int buflen,
                                   int flags, long ifno, OS_SOCKADDR *fromp, int *pFromLen,
                                   int *numBytesRec)

{

    ASSERT( bufp && fromp && pFromLen && numBytesRec );

    /* WARNING: ignoring interface number - not supported on VxWorks??? */
    *numBytesRec = recvfrom( socketDesc, bufp, buflen, flags, (struct sockaddr*) fromp, pFromLen );

    /* Error checking */
    return( checkSocketStatus( *numBytesRec, "bcmOsSocketReceiveFrom" ));
}

/*****************************************************************************
** FUNCTION:   bcmOsSocketSendTo
**
** PURPOSE:    send a message to a socket
**
** PARAMETERS:
**             socketDesc     - socket to receive from
**             bufp           - pointer to data buffer
**             buflen         - length of buffer
**             flags          - flags to underlying protocols
**             ifno           - specifies outgoing interface for message
**                               (0 for default interface)
**             top            - recipient's address
**             toLen          - length of to sockaddr
**             numBytesSent   - number of bytes sent
**
** RETURNS:    OS_STATUS_OK  - success
**             error code    - failure
**
** NOTE:
**
******************************************************************************/
OS_STATUS bcmOsSocketSendTo ( int socketDesc, char *bufp, int buflen, int flags,
                              long ifno, OS_SOCKADDR *top, int toLen, int *numBytesSent )
{
    OS_SOCKADDR *osName;

    ASSERT( bufp && top && numBytesSent );

    /* update OS-independent parameters to OS-dependent values */
    osName = top;
    if ( top->sin_family == OS_AF_INET )
    {
        osName->sin_family = (short)bcmOsConvertSocketOptions((UINT32)top->sin_family);
    }
    //   printf(".k   bcmOsSocketSendTo: send buflen =  (%d)\n",buflen);
    //buflen = 501;
    //bufp[500] = '\0';
    //printf(".k   bcmOsSocketSendTo: trunk send bytes to (%d)\n",buflen);
    //printf(".k   bcmOsSocketSendTo: flags = (%d)\n",flags);

    *numBytesSent = sendto( socketDesc, bufp, buflen, flags, (struct sockaddr_in*) osName, toLen);
    LIBDEBUG(".k   bcmOsSocketSentTo: sent (%d) bytes\n",*numBytesSent);
    /* Error checking */
    return( checkSocketStatus((NATIVE_SOCKET_STATUS) *numBytesSent, "bcmOsSocketSendTo" ));

    //*numBytesSent = sendto( socketDesc, bufp, buflen, flags, (struct sockaddr_in*) osName, toLen);

}

/*****************************************************************************
** FUNCTION:   bcmOsSocketSetOptions
**
** PURPOSE:    set socket options
**
** PARAMETERS:
**             socketDesc    - target socket
**             level         - protocol level of option
**             optname       - option name
**             optval        - pointer to option value
**             optlen        - option length
**
** RETURNS:    OS_STATUS_OK  - success
**             error code    - failure
**
** NOTE:
**
******************************************************************************/
#if CH_NEEDED
OS_STATUS bcmOsSocketSetOptions ( int socketDesc, OS_SOCKOPT level, OS_SOCKOPT optname,
                                  char * optval, int optlen )
{
    // NATIVE_SOCKET_STATUS status;
    // UINT32               osLevel;
    // UINT32               osOption;
    //
    // ASSERT( optval );
    //
    // /* convert form OS-independent values to OS-dependent values */
    // osLevel = bcmOsConvertSocketOptions(level);
    // osOption = bcmOsConvertSocketOptions(optname);
    //
    // /* There does not appear to be a standard set of options, specified by
    //  * the 'osOption', for different operating systems. All operating systems
    //  * appear to have the SO_REUSEADDR option; however, only pSOS has
    //  * SO_REUSEPORT. Therefore, if SO_REUSEADDR is specified and the native
    //  * operating system is pSOS, the SO_REUSEPORT option will also be set. */
    //
    // status = setsockopt( socketDesc, osLevel, osOption, optval, optlen );
    //
    ///* Error checking */
    //return ( checkSocketStatus( status, "bcmOsSocketSetOptions" ));
    int Optval = 1;

    LIBDEBUG(" bcmOsSocketSetOptions was called but is returning without doing anything!!!!\n");
    //   setsockopt (socketDesc, SOL_SOCKET, SO_REUSEADDR, (char *)&Optval, sizeof (int));
    //   setsockopt (socketDesc, SOL_SOCKET, SO_REUSEPORT, (char *)&Optval, sizeof (int));
    return 0;
}
#endif


/*****************************************************************************
** FUNCTION:   bcmOsSocketClose
**
** PURPOSE:    close socket
**
** PARAMETERS:
**             socketDesc    - target socket
**
** RETURNS:    OS_STATUS_OK  - success
**             error code    - failure
**
** NOTE:
**
******************************************************************************/
OS_STATUS bcmOsSocketClose ( int socketDesc )
{
    NATIVE_SOCKET_STATUS status;

    status = close ( socketDesc );

    /* Error checking */
    return( checkSocketStatus( status, "bcmOsSocketClose" ));
}

/*****************************************************************************
** FUNCTION:   bcmOsSocketCloseShared
**
** PURPOSE:    Close all sockets that were created by other tasks, but shared
**             with the calling task.
**
** PARAMETERS: none
**
** RETURNS:    OS_STATUS_OK  - success
**             error code    - failure
**
** NOTE:
**
******************************************************************************/
#if CH_NEEDED
OS_STATUS bcmOsSocketCloseShared ( void )
{
    /* Socket sharing is not required for VxWorks. Therefore, there is
     * nothing to close. */
    return( OS_STATUS_OK );
}
#endif

/*****************************************************************************
** FUNCTION:   bcmOsGetOwnIpAddress()
**
** PURPOSE:    Retrieves the IP address of a given IP stack number (interface)
**
** PARAMETERS: netIface [IN]  - IP Interface Number
**             ipAddr [OUT]   - IP address corresponding to 'netIface' (in
**                               host-byte order)
**
** RETURNS:   OS_STATUS_OK or OS_STATUS_ERR
**
** NOTE:
*****************************************************************************/
#if CH_NEEDED
OS_STATUS bcmOsGetOwnIpAddress( int netIface, IPADDR *ipAddr )
{
    NATIVE_SOCKET_STATUS status = ERROR;

    /* Error checking */
    return( checkSocketStatus( status, "bcmOsGetOwnIpAddress" ));
}
#endif
/*****************************************************************************
** FUNCTION:   checkSocketStatus
**
** PURPOSE:    This function takes the status returned by an OS-specific socket API
**             function call. It maps this to a generic BcmOS return status, and
**             logs any errors.
**
** PARAMETERS: status   - operating system specific status returned from system call
**             fnc      - name of calling function, used for error LOG statements
**
** RETURNS:    Operating system independent status value of type OS_STATUS
**             (that corresponds to 'status') . BcmOS public interface functions
**             should return OS independent status values. Native OS status
**             values should be encapsulated; however, they may be LOGed for
**             debug purposes.
**
** NOTES:      This function is used to validate all OS-specific socket-related API
**             calls invoked by the BcmOS layer. However, not all OS API calls return
**             consistent status values/error codes. Therefore, the following assumptions
**             have been made:
**
**             PSOS:    an error is indicated by a 'status' value of -1
**             VXWORKS: an error is indicated by a 'status' value of ERROR
**             WIN32:   an error is indicated by a 'status' value of INVALID_SOCKET
**                      or SOCKET_ERROR
**
**             These assumptions should handle most OS-specific API calls. However, it
**             is the responsibility of the caller to massage the input parameters to conform
**             to these assumptions for any OS API calls that do not follow these rules.
**
*****************************************************************************/
OS_STATUS checkSocketStatus( NATIVE_SOCKET_STATUS status, char *fnc )
{
    int      err;
    BOOL     bErrStatus;

    /* Get OS-dependent error value and status */
    /* Most VxWorks functions return ERROR when they detect an error (or NULL in
     * the case of functions returning pointers). In addition, they set an error
     * status that elaborates upon the nature of the error. This can be retrieved
     * from the errno variable. */
    err = errno;
    bErrStatus = ( status == ERROR );

    /* Log errors (if any), and return OS-independent status value */
    if ( bErrStatus )
    {
        LOG(( "Error returned by %s - 0x%x\n", fnc, err ));
        return( OS_STATUS_ERR );
    }
    return( OS_STATUS_OK );

} /* end of checkSocketStatus() */

