//*************************************************************************                      
//
//    Copyright 1999  Broadcom Corporation
//    All Rights Reserved
//    No portions of this material may be reproduced in any form without the
//    written permission of:
//             Broadcom Corporation
//             16251 Laguna Canyon Road
//             Irvine, California  92618
//    All information contained in this document is Broadcom Corporation
//    company private, proprietary, and trade secret.
//
//
//
//**************************************************************************
/**++

Module Name:

     cblmodem.h

Abstract:

     This file defines the data types and function prototypes used by
     the CM MAC and the CM HAL modules of the Cable Modem driver.

--**/


#ifndef   CBLMODEM_H
#define   CBLMODEM_H

#if !defined(MAC_STUB) && !defined(WIN32)
#include <bcmos.h>
#endif
#include <bcmtypes.h>

#if defined __cplusplus
extern "C" {
#endif

/********************************************************************************
*    CM_STATUS and its values. CM_STATUS is used by all the private CM HAL and
*    CM MAC functions to return status.
*
********************************************************************************/

typedef ULONG                             CM_STATUS, *PCM_STATUS;

/*
 * CM_STATUS values
 *
 *   Status values are 32 bit values laid out as follows:
 *
 *   3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
 *   1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
 *  +---+-+-------------------------+-------------------------------+
 *  |Sev|C|       Facility          |               Code            |
 *  +---+-+-------------------------+-------------------------------+
 *
 *  where
 *
 *    Sev - is the severity code
 *      00 - Success
 *      01 - Informational
 *      10 - Warning
 *      11 - Error
 *
 *    C - is the Customer code flag
 *
 *    Facility - is the facility code
 *
 *    Code - is the facility's status code
 */

#define STATUS_UNSUCCESSFUL                  ((CM_STATUS)0xC0000001L)
#define STATUS_SUCCESS                       ((CM_STATUS)0x00000000L)
#ifndef STATUS_PENDING
  // This is in WINNT.h; don't re-define if compiling on NT
#define STATUS_PENDING                       ((CM_STATUS)0x00000103L)
#endif
#define STATUS_BUFFER_OVERFLOW               ((CM_STATUS)0xC0000104L)
#define STATUS_INSUFFICIENT_RESOURCES        ((CM_STATUS)0x80000105L)
#define STATUS_NOT_SUPPORTED                 ((CM_STATUS)0x40000106L)

#define CM_STATUS_SUCCESS                    ((CM_STATUS)STATUS_SUCCESS)
#define CM_STATUS_PENDING                    ((CM_STATUS)STATUS_PENDING)
#define CM_STATUS_NOT_RECOGNIZED             ((CM_STATUS)0x00010001L)
#define CM_STATUS_NOT_COPIED                 ((CM_STATUS)0x00010002L)
#define CM_STATUS_NOT_ACCEPTED               ((CM_STATUS)0x00010003L)
#define CM_STATUS_CALL_ACTIVE                ((CM_STATUS)0x00010007L)

#define CM_STATUS_ONLINE                     ((CM_STATUS)0x40010003L)
#define CM_STATUS_RESET_START                ((CM_STATUS)0x40010004L)
#define CM_STATUS_RESET_END                  ((CM_STATUS)0x40010005L)

#define CM_STATUS_MEDIA_CONNECT              ((CM_STATUS)0x4001000BL)
#define CM_STATUS_MEDIA_DISCONNECT           ((CM_STATUS)0x4001000CL)
#define CM_STATUS_HARDWARE_LINE_UP           ((CM_STATUS)0x4001000DL)
#define CM_STATUS_HARDWARE_LINE_DOWN         ((CM_STATUS)0x4001000EL)
#define CM_STATUS_INTERFACE_UP               ((CM_STATUS)0x4001000FL)
#define CM_STATUS_INTERFACE_DOWN             ((CM_STATUS)0x40010010L)
#define CM_STATUS_MEDIA_BUSY                 ((CM_STATUS)0x40010011L)

#define CM_STATUS_UCD_IGNORED                ((CM_STATUS)0x40010012L)
#define CM_STATUS_RNG_RSP_IGNORED            ((CM_STATUS)0x40010013L)
#define CM_STATUS_BPI_IGNORED                ((CM_STATUS)0x40010014L)
#define CM_STATUS_COS_IGNORED                ((CM_STATUS)0x40010015L)

#define CM_STATUS_NOT_RESETTABLE             ((CM_STATUS)0x80010001L)
#define CM_STATUS_SOFT_ERRORS                ((CM_STATUS)0x80010003L)
#define CM_STATUS_HARD_ERRORS                ((CM_STATUS)0x80010004L)
#define CM_STATUS_BUFFER_OVERFLOW            ((CM_STATUS)STATUS_BUFFER_OVERFLOW)

#define CM_STATUS_FAILURE                    ((CM_STATUS)STATUS_UNSUCCESSFUL)
#define CM_STATUS_RESOURCES                  ((CM_STATUS)STATUS_INSUFFICIENT_RESOURCES)
#define CM_STATUS_NOT_SUPPORTED              ((CM_STATUS)STATUS_NOT_SUPPORTED)
#define CM_STATUS_CLOSING                    ((CM_STATUS)0xC0010002L)
#define CM_STATUS_BAD_VERSION                ((CM_STATUS)0xC0010004L)
#define CM_STATUS_BAD_CHARACTERISTICS        ((CM_STATUS)0xC0010005L)
#define CM_STATUS_ADAPTER_NOT_FOUND          ((CM_STATUS)0xC0010006L)
#define CM_STATUS_OPEN_FAILED                ((CM_STATUS)0xC0010007L)
#define CM_STATUS_DEVICE_FAILED              ((CM_STATUS)0xC0010008L)
#define CM_STATUS_MULTICAST_FULL             ((CM_STATUS)0xC0010009L)
#define CM_STATUS_MULTICAST_EXISTS           ((CM_STATUS)0xC001000AL)
#define CM_STATUS_MULTICAST_NOT_FOUND        ((CM_STATUS)0xC001000BL)
#define CM_STATUS_REQUEST_ABORTED            ((CM_STATUS)0xC001000CL)
#define CM_STATUS_RESET_IN_PROGRESS          ((CM_STATUS)0xC001000DL)
#define CM_STATUS_CLOSING_INDICATING         ((CM_STATUS)0xC001000EL)
#define CM_STATUS_INVALID_PACKET             ((CM_STATUS)0xC001000FL)
#define CM_STATUS_OPEN_LIST_FULL             ((CM_STATUS)0xC0010010L)
#define CM_STATUS_ADAPTER_NOT_READY          ((CM_STATUS)0xC0010011L)
#define CM_STATUS_ADAPTER_NOT_OPEN           ((CM_STATUS)0xC0010012L)
#define CM_STATUS_NOT_INDICATING             ((CM_STATUS)0xC0010013L)
#define CM_STATUS_INVALID_LENGTH             ((CM_STATUS)0xC0010014L)
#define CM_STATUS_INVALID_DATA               ((CM_STATUS)0xC0010015L)
#define CM_STATUS_BUFFER_TOO_SHORT           ((CM_STATUS)0xC0010016L)
#define CM_STATUS_INVALID_OID                ((CM_STATUS)0xC0010017L)
#define CM_STATUS_ADAPTER_REMOVED            ((CM_STATUS)0xC0010018L)
#define CM_STATUS_UNSUPPORTED_MEDIA          ((CM_STATUS)0xC0010019L)
#define CM_STATUS_GROUP_ADDRESS_IN_USE       ((CM_STATUS)0xC001001AL)
#define CM_STATUS_FILE_NOT_FOUND             ((CM_STATUS)0xC001001BL)
#define CM_STATUS_ERROR_READING_FILE         ((CM_STATUS)0xC001001CL)
#define CM_STATUS_ALREADY_MAPPED             ((CM_STATUS)0xC001001DL)
#define CM_STATUS_RESOURCE_CONFLICT          ((CM_STATUS)0xC001001EL)
#define CM_STATUS_NO_CABLE                   ((CM_STATUS)0xC001001FL)

#define CM_STATUS_INVALID_SAP                ((CM_STATUS)0xC0010020L)
#define CM_STATUS_SAP_IN_USE                 ((CM_STATUS)0xC0010021L)
#define CM_STATUS_INVALID_ADDRESS            ((CM_STATUS)0xC0010022L)
#define CM_STATUS_VC_NOT_RESERVED            ((CM_STATUS)0xC0010023L)
#define CM_STATUS_DEST_OUT_OF_ORDER          ((CM_STATUS)0xC0010024L)
#define CM_STATUS_VC_NOT_AVAILABLE           ((CM_STATUS)0xC0010025L)
#define CM_STATUS_CELLRATE_NOT_AVAILABLE     ((CM_STATUS)0xC0010026L)
#define CM_STATUS_INCOMPATABLE_QOS           ((CM_STATUS)0xC0010027L)

#define CM_STATUS_MESSGQ_CREATE_FAILED       ((CM_STATUS)0xC0010028L)
#define CM_STATUS_MESSGQ_WRITE_FAILED        ((CM_STATUS)0xC0010029L)
#define CM_STATUS_MESSGQ_READ_FAILED         ((CM_STATUS)0xC0010030L)

#define CM_STATUS_CREATE_THREAD_FAILED       ((CM_STATUS)0xC0010031L)

#define CM_STATUS_INVALID_ATTR_LEN           ((CM_STATUS)0xC0010032L)
#define CM_STATUS_REQD_ATTR_MISSING          ((CM_STATUS)0xC0010033L)
#define CM_STATUS_INVALID_AUTH_KEY_SEQ_NUM   ((CM_STATUS)0xC0010034L)
#define CM_STATUS_INVALID_TEK_SID            ((CM_STATUS)0xC0010035L)


/********************************************************************************
*    CM_HAL_OID and its values
*
********************************************************************************/

/*
 * CM_HAL_OID values
 *   OID values are 32 bit values laid out as follows:
 *
 *   3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
 *   1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
 *  +-+-----------------------------+-------------------------------+
 *  |M|          Unused             |               OID             |
 *  +-+-----------------------------+-------------------------------+
 *
 *  where
 *
 *      M - the multiple instances flag
 *        0 - Only one instance of the HAL OID can exist
 *        1 - Multiple instances of the HAL OID can exist
 *
 *      OID - the 16-bit OID value
 *
 */


typedef        UINT                                              CM_HAL_OID;

/* Hardware Intialization Related: */

#define        CM_HAL_OID_PID                                    ((CM_HAL_OID)0x00000001)

#define        CM_HAL_OID_MAC_PROTOCOL_VER_HIGH                  ((CM_HAL_OID)0x00000002)
#define        CM_HAL_OID_MAC_PROTOCOL_VER_LOW                   ((CM_HAL_OID)0x00000003)

#define        CM_HAL_OID_MAX_UPSTREAM_SIDS                      ((CM_HAL_OID)0x00000004)

/* Packet Filtering Related */

#define        CM_HAL_OID_MAX_DNSTREAM_MSG_DA_UNICAST_FILTERS    ((CM_HAL_OID)0x00000005)
#define        CM_HAL_OID_MAX_DNSTREAM_MSG_DA_MULTICAST_FILTERS  ((CM_HAL_OID)0x00000006)

#define        CM_HAL_OID_MAX_DNSTREAM_PDU_DA_UNICAST_FILTERS    ((CM_HAL_OID)0x00000007)
#define        CM_HAL_OID_MAX_DNSTREAM_PDU_DA_MULTICAST_FILTERS  ((CM_HAL_OID)0x00000008)

#define        CM_HAL_OID_DNSTREAM_MSG_UNICAST_DA                ((CM_HAL_OID)0x80000009)
#define        CM_HAL_OID_DNSTREAM_MSG_MULTICAST_DA              ((CM_HAL_OID)0x8000000A)

#define        CM_HAL_OID_DNSTREAM_PDU_UNICAST_DA			        ((CM_HAL_OID)0x8000000B)
#define        CM_HAL_OID_DNSTREAM_PDU_MULTICAST_DA			     ((CM_HAL_OID)0x8000000C)


/* BP Initialization Related: */

#define        CM_HAL_OID_BP_SECURITY_VER_HIGH                   ((CM_HAL_OID)0x0000000D)
#define        CM_HAL_OID_BP_SECURITY_VER_LOW                    ((CM_HAL_OID)0x0000000E)

#define        CM_HAL_OID_UNENCRYPTED_DATA_PDU_LEN               ((CM_HAL_OID)0x0000000F)
#define        CM_HAL_OID_UNENCRYPTED_RSVD_PDU_LEN               ((CM_HAL_OID)0x00000010)
#define        CM_HAL_OID_UNENCRYPTED_ATM_PDU_LEN                ((CM_HAL_OID)0x00000011)
#define        CM_HAL_OID_UPSTREAM_DEFAULT_PRIV_VER              ((CM_HAL_OID)0x00000012)

#define        CM_HAL_OID_MAX_DNSTREAM_DES_SIDS                  ((CM_HAL_OID)0x00000013)
#define        CM_HAL_OID_MAX_UPSTREAM_DES_SIDS                  ((CM_HAL_OID)0x00000014)

/* UCD Related: */

#define         CM_HAL_OID_UPSTREAM_CONFIG                       ((CM_HAL_OID)0x00000015)

/* RNG-REQ Related: */

#define        CM_HAL_OID_RNG_REQ_FIFO                           ((CM_HAL_OID)0x00000016)
#define        CM_HAL_OID_ADJUST_UPSTREAM_POWER                  ((CM_HAL_OID)0x00000017)
#define        CM_HAL_OID_UPSTREAM_POWER_LEVEL                   ((CM_HAL_OID)0x00000018)

/* RNG-RSP Related: */

#define        CM_HAL_OID_PRIMARY_SID                            ((CM_HAL_OID)0x00000019)
#define        CM_HAL_OID_RNG_ADJUST                             ((CM_HAL_OID)0x00000020)

/* REG-RSP Related: */

#define        CM_HAL_OID_UPSTREAM_SID                           ((CM_HAL_OID)0x80000021)

/* BPI TEK KEY-REPLY Related: */

#define        CM_HAL_OID_DNSTREAM_DES_SID                       ((CM_HAL_OID)0x80000022)
#define        CM_HAL_OID_UPSTREAM_DES_SID                       ((CM_HAL_OID)0x80000023)
#define        CM_HAL_OID_TEK_PARAMS                             ((CM_HAL_OID)0x80000024)
#define        CM_HAL_OID_DES_KEY_EVEN                           ((CM_HAL_OID)0x80000025)
#define        CM_HAL_OID_DES_KEY_ODD                            ((CM_HAL_OID)0x80000026)
#define        CM_HAL_OID_DES_IV_EVEN                            ((CM_HAL_OID)0x80000027)
#define        CM_HAL_OID_DES_IV_ODD                             ((CM_HAL_OID)0x80000028)

/* Statistics/Diagnostics Related: */

#define        CM_HAL_OID_DNSTREAM_FIFO_FULL_COUNT               ((CM_HAL_OID)0x00000029)
#define        CM_HAL_OID_DNSTREAM_MSG_OVERRUN_COUNT             ((CM_HAL_OID)0x0000002A)
#define        CM_HAL_OID_DNSTREAM_PDU_OVERRUN_COUNT             ((CM_HAL_OID)0x0000002B)
#define        CM_HAL_OID_DNSTREAM_QUALIFIED_MAP_COUNT           ((CM_HAL_OID)0x0000002C)
#define        CM_HAL_OID_DNSTREAM_QUALIFIED_SYNC_COUNT          ((CM_HAL_OID)0x0000002D)
#define        CM_HAL_OID_DNSTREAM_QUALIFIED_UCD_COUNT           ((CM_HAL_OID)0x0000002E)
#define        CM_HAL_OID_DNSTREAM_REJECTED_MAP_COUNT            ((CM_HAL_OID)0x0000002F)
#define        CM_HAL_OID_DNSTREAM_MPEG_PACKET_ERRORS_COUNT      ((CM_HAL_OID)0x00000030)
#define        CM_HAL_OID_DNSTREAM_MSG_CRC_FAILURE_COUNT         ((CM_HAL_OID)0x00000031)
#define        CM_HAL_OID_DNSTREAM_HCS_FAILURE_COUNT             ((CM_HAL_OID)0x00000032)
#define        CM_HAL_OID_DNSTREAM_PDU_COUNT                     ((CM_HAL_OID)0x00000033)
#define        CM_HAL_OID_DNSTREAM_MSG_COUNT                     ((CM_HAL_OID)0x00000034)
#define        CM_HAL_OID_DNSTREAM_VALID_HDR_COUNT               ((CM_HAL_OID)0x00000035)
#define        CM_HAL_OID_DNSTREAM_MSG_BYTES_COUNT               ((CM_HAL_OID)0x00000036)

#define        CM_HAL_OID_UPSTREAM_PACKET_COUNT                  ((CM_HAL_OID)0x80000037)
#define        CM_HAL_OID_UPSTREAM_UNICAST_MINISLOTS             ((CM_HAL_OID)0x80000038)
#define        CM_HAL_OID_UPSTREAM_CONTENTION_MINISLOTS          ((CM_HAL_OID)0x80000039)
#define        CM_HAL_OID_UPSTREAM_BW_RE_REQ_COUNT               ((CM_HAL_OID)0x8000003A)
#define        CM_HAL_OID_UPSTREAM_BW_REQ_FAILURE_COUNT          ((CM_HAL_OID)0x8000003B)
#define        CM_HAL_OID_UPSTREAM_REQ_DATA_PDU_RETRANSMIT_COUNT ((CM_HAL_OID)0x8000003C)
#define        CM_HAL_OID_UPSTREAM_REQ_DATA_PDU_FAILURE_COUNT    ((CM_HAL_OID)0x8000003D)
#define        CM_HAL_OID_UPSTREAM_RNG_REQ_COUNT                 ((CM_HAL_OID)0x0000003E)
#define        CM_HAL_OID_UPSTREAM_BYTE_COUNT                    ((CM_HAL_OID)0x0000003F)

/* Miscellaneous: */

#define        CM_HAL_OID_LOST_SYNC_INTERVAL                     ((CM_HAL_OID)0x00000040)
#define        CM_HAL_OID_MAX_BW_REREQ_COUNT                     ((CM_HAL_OID)0x00000041)
#define        CM_HAL_OID_CONTENTION_DATA_LEN                    ((CM_HAL_OID)0x00000042)

#define        CM_HAL_OID_UPSTREAM_GROUPCAST_SID                 ((CM_HAL_OID)0x80000043)

#define        CM_HAL_OID_UPSTREAM_FWD_LOOKUP_BASE_OFFSET        ((CM_HAL_OID)0x00000044)
#define        CM_HAL_OID_UPSTREAM_REVERSE_LOOKUP_BASE_OFFSET    ((CM_HAL_OID)0x00000045)

/* DOCSIS MIBs related */

#define        CM_HAL_OID_UPSTREAM_CHANNEL_WIDTH                 ((CM_HAL_OID)0x80000046)
#define        CM_HAL_OID_UPSTREAM_CHANNEL_TX_TIMING_OFFSET      ((CM_HAL_OID)0x80000047)
#define        CM_HAL_OID_UPSTREAM_CHANNEL_TX_POWER              ((CM_HAL_OID)0x80000048)

#define        CM_HAL_OID_DNSTREAM_CODEWORDS_UNERRORED           ((CM_HAL_OID)0x00000049)
#define        CM_HAL_OID_DNSTREAM_CODEWORDS_CORRECTED           ((CM_HAL_OID)0x0000004A)
#define        CM_HAL_OID_DNSTREAM_CODEWORDS_UNCORRECTABLE       ((CM_HAL_OID)0x0000004B)
#define        CM_HAL_OID_DNSTREAM_SIGNAL_TO_NOISE_RATIO         ((CM_HAL_OID)0x0000004C)
#define        CM_HAL_OID_DNSTREAM_TOTAL_MICROREFLECTIONS        ((CM_HAL_OID)0x0000004D)

#define        CM_HAL_OID_TX_MINISLOTS_IMMEDIATE_MODE            ((CM_HAL_OID)0x8000004E)
#define        CM_HAL_OID_TX_MINISLOTS_DEDICATED_MODE            ((CM_HAL_OID)0x8000004F)
#define        CM_HAL_OID_TX_DATA_PDU_RETRIES                    ((CM_HAL_OID)0x80000050)
#define        CM_HAL_OID_TX_DATA_PDU_FAILURES                   ((CM_HAL_OID)0x80000051)
#define        CM_HAL_OID_TX_BW_REQ_RETRIES                      ((CM_HAL_OID)0x80000052)
#define        CM_HAL_OID_TX_BW_REQ_FAILURES                     ((CM_HAL_OID)0x80000053)

#define        CM_HAL_OID_DNSTREAM_EQUALIZATION_DATA             ((CM_HAL_OID)0x00000054)
#define        CM_HAL_OID_UPSTREAM_EQUALIZATION_DATA             ((CM_HAL_OID)0x00000055)

#define        CM_HAL_OID_DNSTREAM_CODEWORDS_TOTAL               ((CM_HAL_OID)0x00000056)

/* DOCSIS 1.1 extensions */                                       
               //-----------------------------------------
               // Query/Set accepts PUPSTREAM_SERVICE_FLOW
               //-----------------------------------------
               // Query only
#define        CM_HAL_OID_PRIMARY_UPSTREAM_SERVICE_FLOW          ((CM_HAL_OID)0x00000057)
               // Query/Set  (hUsService=0 allocates a new BcmHandle, hUservice > 0 returns info for BcmHandle)
#define        CM_HAL_OID_UPSTREAM_SERVICE_FLOW                  ((CM_HAL_OID)0x00000058)
               // Set only
#define        CM_HAL_OID_RELEASE_UPSTREAM_SERVICE_FLOW          ((CM_HAL_OID)0x00000059)
#define        CM_HAL_OID_ACTIVATE_UPSTREAM_SERVICE_FLOW         ((CM_HAL_OID)0x0000005A)
#define        CM_HAL_OID_DEACTIVATE_UPSTREAM_SERVICE_FLOW       ((CM_HAL_OID)0x0000005B)
               // Query/Set
#define        CM_HAL_OID_CONFIG_UPSTREAM_SERVICE_FLOW           ((CM_HAL_OID)0x0000005C)

/* Security Related */
#define        CM_HAL_OID_SECURE_PROCESS_HEADER                   ((CM_HAL_OID)0x0000005D)
#define        CM_HAL_OID_SECURE_PROCESS_DATA                     ((CM_HAL_OID)0x0000005E)

/* Used in the TCPIP Hal for XID/TEST support.*/
#define		   CM_HAL_OID_GET_ETHPACKET                           ((CM_HAL_OID)0x0000005F)


/* Downstream Channel Related: */ 

// accepts DownstreamChannelStatus object (Query Only)
#define        CM_HAL_OID_DNSTREAM_CHANNEL_STATUS                 ((CM_HAL_OID)0x00000060)
// accepts ulParam = dsFreq, returns pvInfoBuffer = *channel
#define        CM_HAL_OID_CONVERT_DOWNSTREAM_FREQUENCY_TO_CHANNEL ((CM_HAL_OID)0x00000061)

#define        CM_HAL_OID_UPSTREAM_TEST                           ((CM_HAL_OID)0x00000062)
#define        CM_HAL_OID_UPSTREAM_SWEEP_TEST                     ((CM_HAL_OID)0x00000063)

// accepts DOWNSTREAM_CHANNEL_SETTINGS object = pvInfoBuffer (Set/Query)
#define        CM_HAL_OID_DNSTREAM_CONFIG                         ((CM_HAL_OID)0x00000064)

// accepts: ulParam=currentDsFreq, pvInfoBuffer=*nextDsFreq, pvInfoBufferLen=*sizeof nextDsFreq (ie: 4)
#define        CM_HAL_OID_GET_NEXT_DOWNSTREAM_FREQUENCY           ((CM_HAL_OID)0x00000065)


#define        CM_HAL_OID_DOWNSTREAM_TEST                         ((CM_HAL_OID)0x0000006F)

/* DOCSIS 1.1 extensions, continued */                                       
               //-----------------------------------------
               // Query/Set accepts PUPSTREAM_SERVICE_FLOW
               //-----------------------------------------
               // Set only
#define        CM_HAL_OID_FLUSH_UPSTREAM_SERVICE_FLOW             ((CM_HAL_OID) 0x00000070)
               //--------------------------------------------------------
               // Query only, accepts: 
               // ulParam - BcmHandle hUsServiceFlow  (NULL = all flows)
               // pvInfoBuffer -  (uint32 *) returns queued packet count
               //--------------------------------------------------------
#define        CM_HAL_OID_UPSTREAM_SERVICE_FLOW_HW_PACKETS_QUEUED ((CM_HAL_OID) 0x00000071)

/* Upstream Channel Related: */ 

// Query Only
// pvInfoBuffer -  (uint32 *) returns number equalizer taps supported by CM HAL
#define        CM_HAL_OID_UPSTREAM_NUM_EQ_TAPS_SUPPORTED 		  ((CM_HAL_OID) 0x00000072)

// Query Only
// pvInfoBuffer -  (uint32 *) returns number of forward equalizer taps/symb supported CM HAL
#define        CM_HAL_OID_UPSTREAM_NUM_EQ_TAPS_PER_SYMBOL_SUPPORTED  ((CM_HAL_OID) 0x00000073)


/* For debug purposes only.  Reads the current values of the LDAII and LDAIT
   registers, returning them in the struct provided (Query only).  Primarily
   used to support calibration of the downstream power measurement table. */
#define        CM_HAL_OID_READ_LDAIx_REGISTERS                    ((CM_HAL_OID) 0x00000080)

/* For debug purposes only.  Reads (QueryInfo) or writes (SetInfo) the specified
   Multi-byte register in the downstream MAC section.  You must specify a valid
   "read" or "write" opcode in ulParam, and the number of bytes to be read or
   written.  pvInfoBuffer must point to a uint32, but pulInfoBufferLen must be
   the actual number of bytes that you want to read or write. */
#define        CM_HAL_OID_ACCESS_MULTIBYTE_REGISTER               ((CM_HAL_OID) 0x00000081)

// For debug/status purposes only.  Dumps CM HAL internal information Objects
// Set Only 
//
// ulParam - index of object to dump (zero if listing)
// pvInfoBuffer -  (CmHalInternalInfo *) specifies CM HAL information to dump
// ulInfoBufferLen - size of CmHalInternalInfo object
#define        CM_HAL_OID_DUMP_INTERNAL_INFO                      ((CM_HAL_OID) 0x00000082)

               // does not require ulparam (MAC handles where to place
               // new entries (set info only)
#define        CM_HAL_OID_ADD_DNSTREAM_PDU_UNICAST_DA		   ((CM_HAL_OID)0x00000090)
#define        CM_HAL_OID_ADD_DNSTREAM_PDU_MULTICAST_DA		   ((CM_HAL_OID)0x00000091)
#define        CM_HAL_OID_DELETE_DNSTREAM_PDU_MULTICAST_DA	   ((CM_HAL_OID)0x00000092)

// Used for DCC SAID substitution
#define        CM_HAL_OID_SUBSTITUTE_HW_SID                    ((CM_HAL_OID)0x00000098)

// Wrapper for calls to BcmAlloc (QueryInfo).  For now, only supported by the
// CM HAL.
//
// ulParam - the buffer size being requested
// pvInfoBuffer - pointer to the buffer pointer; this is where the buffer address is returned.
// ulInfoBufferLen - must be 4 (sizeof(void *)).
#define        CM_HAL_OID_ALLOC_BUFFER                         ((CM_HAL_OID)0x00000100)

// Wrapper for calls to BcmFree (SetInfo).  For now, only supported by the CM
// HAL.
//
// ulParam - not used.
// pvInfoBuffer - pointer to the buffer to be freed.
// ulInfoBufferLen - must be 4 (sizeof(void *)).
#define        CM_HAL_OID_FREE_BUFFER                          ((CM_HAL_OID)0x00000101)

// lock on the downstream
#define        CM_HAL_OID_LOCK_DS                              ((CM_HAL_OID)0x00000102)

////////////////////////////////////////////////////////////////////////////////
// CM HAL HW PHS support 
//
typedef enum 
{
  kdocsisCmHalUpstreamHwPhs 	=	0x00000001,  // Upstream HW PHS supported
  kdocsisCmHalDownstreamHwPhs 	= 	0x00000002   // Downstream HW PHS supported
} DocsisCmHalHwPhsSupportBitMask;

// Query only
//
// ulParam - N/A
// pvInfoBuffer - pointer to DocsisCmHalHwPhsSupportBitMask or size of HW PHS supported
// pulInfoBufferLen - must be 1 (sizeof(uint8)).

// PHS hardware support (Returns DocsisCmHalHwPhsSupportBitMask)
#define        CM_HAL_HARDWARE_PHS_SUPPORT                     ((CM_HAL_OID)0x00000103)
// PHS hardware support (Returns size of HW PHS supported)
#define        CM_HAL_HARDWARE_PHS_SUPPORT_SIZE                ((CM_HAL_OID)0x00000104)

// Query/Set
//
// ulParam - N/A
// pvInfoBuffer - pointer to DocsisCmHalPhsRule
// ulInfoBufferLen - must be (sizeof(DocsisCmHalPhsRule)).
//
// for Query caller provides:
//   phsDirection
//   hUsServiceFlow (US only)
//   phsIndex
// remaining parameters are returned.
//
#define        CM_HAL_SET_PHS_RULE				               ((CM_HAL_OID)0x00000105)
#define        CM_HAL_DELETE_PHS_RULE  			               ((CM_HAL_OID)0x00000106)

typedef struct _DocsisCmHalPhsRule  {
   long		   	phsDirection;   // kdocsisCmHalUpstreamHwPhs or kdocsisCmHalDownstreamHwPhs
   BcmHandle   	hUsServiceFlow; // applicable for kdocsisCmHalUpstreamHwPhs (N/A for kdocsisCmHalDownstreamHwPhs)
   byte *     	pPhsMask;
   byte *     	pPhsSuppressedString;  		// applicable for kdocsisCmHalDownstreamHwPhs only
   byte  	   	phsSize; 
   byte  		phsIndex;
} DocsisCmHalPhsRule;

/********************************************************************************
*    CM_HAL_MODE and its values
*
********************************************************************************/

/*
 * CM_HAL_MODE values
 *   MODE values are 32 bit values laid out as follows:
 *
 *   3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
 *   1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
 *  +-+-----------------------------+-------------------------------+
 *  |M|          Unused             |              Mode             |
 *  +-+-----------------------------+-------------------------------+
 *
 *  where
 *
 *      M - the multiple instances flag
 *        0 - Only one instance of the HAL MODE can exist
 *        1 - Multiple instances of the HAL MODE can exist
 *
 *      MODE - the 16-bit MODE value
 *
 */


typedef        UINT                                                        CM_HAL_MODE;

/* Hardware Intialization Related: */

/* Packet Filtering Related */

#define        CM_HAL_MODE_DNSTREAM_MSG_FILTER_PROMISCUOUS                 ((CM_HAL_MODE)0x00000000)
#define        CM_HAL_MODE_DNSTREAM_MSG_FILTER_MAP_TO_MEMORY               ((CM_HAL_MODE)0x00000001)
#define        CM_HAL_MODE_DNSTREAM_MSG_FILTER_TSTAMP_TO_MEMORY            ((CM_HAL_MODE)0x00000002)
#define        CM_HAL_MODE_DNSTREAM_MSG_FILTER_UNCHANGED_UCD_TO_MEMORY     ((CM_HAL_MODE)0x00000003)

#define        CM_HAL_MODE_DNSTREAM_PDU_FILTER_ATM_TO_MEMORY               ((CM_HAL_MODE)0x00000004)
#define        CM_HAL_MODE_DNSTREAM_PDU_FILTER_FULL_SECURITY_TO_MEMORY     ((CM_HAL_MODE)0x00000005)
#define        CM_HAL_MODE_DNSTREAM_PDU_FILTER_BP_TO_MEMORY                ((CM_HAL_MODE)0x00000006)
#define        CM_HAL_MODE_DNSTREAM_PDU_FILTER_NO_SECURITY_TO_MEMORY       ((CM_HAL_MODE)0x00000007)
#define        CM_HAL_MODE_DNSTREAM_PDU_FILTER_SID_PROMISCUOUS             ((CM_HAL_MODE)0x00000008)
#define        CM_HAL_MODE_DNSTREAM_PDU_FILTER_BP_VERSION_PROMISCUOUS      ((CM_HAL_MODE)0x00000009)

#define        CM_HAL_MODE_DNSTREAM_MSG_DA_FILTER_PROMISCUOUS              ((CM_HAL_MODE)0x0000000A)
#define        CM_HAL_MODE_DNSTREAM_MSG_DA_FILTER_BROADCAST                ((CM_HAL_MODE)0x0000000B)
#define        CM_HAL_MODE_DNSTREAM_MSG_DA_FILTER_GENERAL_MULTICAST        ((CM_HAL_MODE)0x0000000C)
#define        CM_HAL_MODE_DNSTREAM_MSG_DA_FILTER_SPECIFIC_MULTICAST       ((CM_HAL_MODE)0x0000000D)

#define        CM_HAL_MODE_DNSTREAM_MSG_DA_FILTER_UNICAST                  ((CM_HAL_MODE)0x8000000E)

#define        CM_HAL_MODE_DNSTREAM_PDU_DA_FILTER_ATM                      ((CM_HAL_MODE)0x0000000F)
#define        CM_HAL_MODE_DNSTREAM_PDU_DA_FILTER_RSVD_PACKETS             ((CM_HAL_MODE)0x00000010)
#define        CM_HAL_MODE_DNSTREAM_PDU_DA_FILTER_BP                       ((CM_HAL_MODE)0x00000011)

#define        CM_HAL_MODE_DNSTREAM_PDU_DA_FILTER_PROMISCUOUS              ((CM_HAL_MODE)0x00000012)
#define        CM_HAL_MODE_DNSTREAM_PDU_DA_FILTER_BROADCAST                ((CM_HAL_MODE)0x00000013)
#define        CM_HAL_MODE_DNSTREAM_PDU_DA_FILTER_GENERAL_MULTICAST        ((CM_HAL_MODE)0x00000014)
#define        CM_HAL_MODE_DNSTREAM_PDU_DA_FILTER_SPECIFIC_MULTICAST       ((CM_HAL_MODE)0x00000015)

#define        CM_HAL_MODE_DNSTREAM_PDU_DA_FILTER_UNICAST                  ((CM_HAL_MODE)0x80000016)

/* BP Initialization Related: */

#define        CM_HAL_MODE_DNSTREAM_BP_ENABLE                              ((CM_HAL_MODE)0x0000001A)
#define        CM_HAL_MODE_DNSTREAM_DES_CBC                                ((CM_HAL_MODE)0x0000001B)

#define        CM_HAL_MODE_UPSTREAM_BP_ENABLE                              ((CM_HAL_MODE)0x0000001C)
#define        CM_HAL_MODE_UPSTREAM_DES_CBC                                ((CM_HAL_MODE)0x0000001D)

/* UCD Related: */

#define        CM_HAL_MODE_UPSTREAM_CHANNEL_ID_VALIDATE                    ((CM_HAL_MODE)0x00000020)

/* Ranging Related: */

#define        CM_HAL_MODE_CLEAR_RNG_REQ_FIFO                              ((CM_HAL_MODE)0x00000021)
#define        CM_HAL_MODE_TRANSMIT_INIT_MAINT_RNG_REQ                     ((CM_HAL_MODE)0x00000022)
#define        CM_HAL_MODE_RETRANSMIT_INIT_MAINT_RNG_REQ                   ((CM_HAL_MODE)0x00000023)
#define        CM_HAL_MODE_TRANSMIT_STN_MAINT_RNG_REQ                      ((CM_HAL_MODE)0x00000024)
#define        CM_HAL_MODE_TRANSMIT_PERIODIC_RNG_REQ                       ((CM_HAL_MODE)0x00000025)

/* BPI TEK KEY-REPLY Related: */

#define        CM_HAL_MODE_DNSTREAM_DES_SID_ENABLE                         ((CM_HAL_MODE)0x80000026)

/* Miscellaneous: */

#define        CM_HAL_MODE_RESET                                           ((CM_HAL_MODE)0x00000030)
#define        CM_HAL_MODE_UPSTREAM_RESET                                  ((CM_HAL_MODE)0x00000031)
#define        CM_HAL_MODE_DNSTREAM_RESET                                  ((CM_HAL_MODE)0x00000032)
#define        CM_HAL_MODE_ENABLE_INTERRUPT                                ((CM_HAL_MODE)0x00000033)

#define        CM_HAL_MODE_UPSTREAM_IGNORE_SYNC                            ((CM_HAL_MODE)0x00000034)
#define        CM_HAL_MODE_UPSTREAM_TRANSMIT_INHIBIT                       ((CM_HAL_MODE)0x00000035)
#define        CM_HAL_MODE_UPSTREAM_TRANSMIT_ENABLE                        ((CM_HAL_MODE)0x00000036)
#define        CM_HAL_MODE_UPSTREAM_PRIMARY_SID_ENABLE                     ((CM_HAL_MODE)0x00000037)

#define        CM_HAL_MODE_UPSTREAM_GROUPCAST_SID_ENABLE                   ((CM_HAL_MODE)0x80000038)

#define        CM_HAL_MODE_UPSTREAM_UNICAST_SID_ENABLE                     ((CM_HAL_MODE)0x80000039)

#define        CM_HAL_MODE_INIT_PHY_FOR_DIAGS                              ((CM_HAL_MODE)0x8000003A)

/* DOCSIS 1.1 extensions */

#define	      CM_HAL_MODE_UPSTREAM_FRAGMENTATION_ENABLE	                  ((CM_HAL_MODE)0x0000003B)
#define	      CM_HAL_MODE_UPSTREAM_DIAG_CW                  	          ((CM_HAL_MODE)0x0000003C)
#define	      CM_HAL_MODE_UPSTREAM_DIAG_23_BITS                  	      ((CM_HAL_MODE)0x0000003D)
#define	      CM_HAL_MODE_UPSTREAM_DIAG_BURST                    	      ((CM_HAL_MODE)0x0000003E)

/* Miscellaneous: */
                                                                           
#define	      CM_HAL_MODE_ENABLE_INTERFACE                  	            ((CM_HAL_MODE)0x0000003F)
#define	      CM_HAL_MODE_REINITIALIZE_INTERFACE            	            ((CM_HAL_MODE)0x00000040)

/* BPI TEK KEY-REPLY Related: */

#define        CM_HAL_MODE_TEK_VALID                                       ((CM_HAL_MODE)0x00000041)

/* Packet Filtering Related */

#define        CM_HAL_MODE_CLR_DNSTREAM_PDU_DA_FILTER                      ((CM_HAL_MODE)0x00000042)

#if (POWER_MGMT)
/* power management */
#define        CM_HAL_MODE_POWERDOWN                                       ((CM_HAL_MODE)0x00000043)
#endif


#define        CM_HAL_MODE_CAM_INIT				                           ((CM_HAL_MODE)0x00000044)

/* Diagnostics Related */

#define     CM_HAL_MODE_FLUSH_ALL_UPSTREAM_SERVICE_FLOWS                    ((CM_HAL_MODE) 0x00000054)
#define     CM_HAL_MODE_DUMP_MIB                                            ((CM_HAL_MODE) 0x00000055)  

/* NIHAL Related */

#define		CM_HAL_MODE_SET_EMAC_CAM_REJECT									((CM_HAL_MODE) 0x00000056)  

/* DOCSIS 1.1 extensions (continued) */
// Query/Set
// Inhibits queuing of packets to indicated service flow 
// accepts: ulParam = BcmHandle hUsServiceFlow (==NULL for all flows)                        
//          bool bflag - inhibit (TRUE/FALSE
#define	   CM_HAL_MODE_UPSTREAM_SERVICE_FLOW_QUEUE_INHIBIT                	((CM_HAL_MODE)0x00000057)


/* DOCSIS 2.0 (Advanced Phy) extensions */

// Query only
// Indicates whether or not CM HAL supports DOCSIS 2.0
// bool pbflag - TRUE/FALSE for setting modem capabilities
#define	   CM_HAL_MODE_ADVANCED_PHY_SUPPORT					                ((CM_HAL_MODE)0x00000058)

// Query/Set
// Set - Sets the CM HAL for advanced phy operation (if supported)
// Query - Returns whether or not CM HAL is in advanced phy mode
// bool bflag - TRUE/FALSE
#define	   CM_HAL_MODE_ADVANCED_PHY_OPERATION			    	            ((CM_HAL_MODE)0x00000059)

#define	      CM_HAL_MODE_UPSTREAM_DEBUG                    	         	((CM_HAL_MODE)0x0000005A)
#define	      CM_HAL_MODE_START_SCAN                        	         	((CM_HAL_MODE)0x0000005B)

////////////////////////////////////////////////////////////////////////////////
// CM HAL HW PHS support 
//

// Set only
// Deletes all configured PHS rules (upstream and downstream)
// bool bflag - TRUE
#define        CM_HAL_MODE_DELETE_ALL_PHS_RULES		               		   	((CM_HAL_MODE)0x0000005C)

// Query/Set
// Enable/Disable PHS if HW supports it or query state of
// HW PHS support
#define        CM_HAL_MODE_PHS_ENABLE 			  	               		   	((CM_HAL_MODE)0x0000005D)

/********************************************************************************
*    CM_MAC_OID and its values
*
********************************************************************************/

/*
 * CM_MAC_OID values
 *   OID values are 32 bit values laid out as follows:
 *
 *   3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
 *   1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
 *  +-+-----------------------------+-------------------------------+
 *  |M|          Unused             |               OID             |
 *  +-+-----------------------------+-------------------------------+
 *
 *  where
 *
 *      M - the multiple instances flag
 *        0 - Only one instance of the MAC OID can exist
 *        1 - Multiple instances of the MAC OID can exist
 *
 *      OID - the 16-bit OID value
 *
 */

typedef        UINT                                              CM_MAC_OID;

#define        CM_MAC_OID_DNSTREAM_SCAN                          ((CM_MAC_OID)0x00000000)
#define        CM_MAC_OID_PULSE_LOST                             ((CM_MAC_OID)0x00000001)
#define        CM_MAC_OID_SYNC_LOST                              ((CM_MAC_OID)0x00000002)
#define        CM_MAC_OID_RNG_REQ_SENT                           ((CM_MAC_OID)0x00000003)
#define        CM_MAC_OID_MAP_UCD_CHANGE_DETECTED                ((CM_MAC_OID)0x00000004)

#define        CM_MAC_OID_LOST_FEC_LOCK                          ((CM_MAC_OID)0x00000005)
#define        CM_MAC_OID_BANDWIDTH_REQUEST_FAILED               ((CM_MAC_OID)0x00000006)

#define        CM_MAC_OID_INITIAL_MAINT_REGION_DETECTED          ((CM_MAC_OID)0x00000007)

/* BPI Related: */

#define        CM_MAC_OID_DES_KEY_SEQ_MISMATCH                   ((CM_MAC_OID)0x00000008)

// UCD change update for cached UCD channels
#define        CM_MAC_OID_MAP_UCD_CHANGE_UPDATE                  ((CM_MAC_OID)0x00000009)


/********************************************************************************
*    BOARD_HAL_MODE and its values
*
********************************************************************************/

/*
 * BOARD_HAL_MODE values
 *   MODE values are 32 bit values laid out as follows:
 *
 *   3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
 *   1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
 *  +-+-----------------------------+-------------------------------+
 *  |M|          Unused             |              MODE             |
 *  +-+-----------------------------+-------------------------------+
 *
 *  where
 *
 *      M - the multiple instances flag
 *        0 - Only one instance of the BOARD HAL MODE can exist
 *        1 - Multiple instances of the BOARD HAL MODE can exist
 *
 *      MODE - the 16-bit MODE value
 *
 */

typedef        UINT                                              BOARD_HAL_MODE;
			   
#define        BOARD_HAL_MODE_ENABLE_WATCHDOG                    ((BOARD_HAL_MODE)0x00000000)
#define        BOARD_HAL_MODE_PAUSE_WATCHDOG                     ((BOARD_HAL_MODE)0x00000001)
#define        BOARD_HAL_MODE_ENABLE_READY_LED                   ((BOARD_HAL_MODE)0x00000002)
#define   	   BOARD_HAL_MODE_ENABLE_SYNC_LED                    ((BOARD_HAL_MODE)0x00000003)
#define        BOARD_HAL_MODE_ENABLE_DOWNSTREAM_LED              ((BOARD_HAL_MODE)0x00000004)
#define        BOARD_HAL_MODE_ENABLE_TEST_LED                    ((BOARD_HAL_MODE)0x00000005)
#define        BOARD_HAL_MODE_ENABLE_USB_LED                     ((BOARD_HAL_MODE)0x00000006)
#define        BOARD_HAL_MODE_ENABLE_WLAN_LED                    ((BOARD_HAL_MODE)0x00000007)
#define        BOARD_HAL_MODE_ENABLE_RANGED_LED                  ((BOARD_HAL_MODE)0x00000008)

/********************************************************************************
*    BOARD_HAL_OID and its values
*
********************************************************************************/

/*
 * BOARD_HAL_OID values
 *   OID values are 32 bit values laid out as follows:
 *
 *   3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
 *   1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
 *  +-+-----------------------------+-------------------------------+
 *  |M|          Unused             |               OID             |
 *  +-+-----------------------------+-------------------------------+
 *
 *  where
 *
 *      M - the multiple instances flag
 *        0 - Only one instance of the MAC OID can exist
 *        1 - Multiple instances of the MAC OID can exist
 *
 *      OID - the 16-bit OID value
 *
 */

typedef        UINT                                              BOARD_HAL_OID;

// Set only
//
// ulParam = "don't care"
// pvInfoBuffer = buffer (starting address)
// ulInfoBufferLen = length
//
#define		   BOARD_HAL_OID_INVALIDATE_DCACHE_BY_ADDRESS  		((BOARD_HAL_OID)0x00000000)
#define		   BOARD_HAL_OID_FLUSH_DCACHE_BY_ADDRESS       		((BOARD_HAL_OID)0x00000001)

// (note: valid dirty cache lines are committed to memory if in write-back mode)
#define		   BOARD_HAL_OID_CLEAR_DCACHE_BY_INDEX  			((BOARD_HAL_OID)0x00000002)

// ulParam = "don't care"
// pvInfoBuffer = PBoardHalPeriodicTimerConfig
// ulInfoBufferLen = "don't care"
#define		   BOARD_HAL_OID_INSTALL_PERIODIC_TIMER				((BOARD_HAL_OID)0x00000003)
#define		   BOARD_HAL_OID_UNINSTALL_PERIODIC_TIMER	  		((BOARD_HAL_OID)0x00000004)
#define		   BOARD_HAL_OID_START_PERIODIC_TIMER	  			((BOARD_HAL_OID)0x00000005)
#define		   BOARD_HAL_OID_SUSPEND_PERIODIC_TIMER		 	   	((BOARD_HAL_OID)0x00000006)

typedef struct _BoardHalPeriodicTimerConfig {
   uint32 	    (*pfIrqHandler)(BcmHandle);	// pointer to client IRQ Handler routine (INSTALL)
   uint32 		periodMs;					// period for interrupt in ms  (INSTALL,START)
   BcmHandle   	hTimer;						// handle for Timer to service (INSTALL,UNINSTALL,START,SUSPEND)
}BoardHalPeriodicTimerConfig, *PBoardHalPeriodicTimerConfig;

/********************************************************************************
*    NI_HAL_OID and its values
*
********************************************************************************/

/*
 * NI_HAL_OID values
 *   OID values are 32 bit values laid out as follows:
 *
 *   3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
 *   1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
 *  +-+-----------------------------+-------------------------------+
 *  |M|          Unused             |               OID             |
 *  +-+-----------------------------+-------------------------------+
 *
 *  where
 *
 *      M - the multiple instances flag
 *        0 - Only one instance of the MAC OID can exist
 *        1 - Multiple instances of the MAC OID can exist
 *
 *      OID - the 16-bit OID value
 *
 */

typedef        UINT                                              NI_HAL_OID;
typedef        UINT                                              NI_HAL_MODE;

#define		   NI_HAL_OID_ENET_LINK_SPEED       				   ((NI_HAL_OID)0x00000000)
#define		   NI_HAL_OID_ENET_LINK_DUPLEX       				   ((NI_HAL_OID)0x00000001)
#define		   NI_HAL_OID_ENET_LINK_AUTONEG_ENABLE 				((NI_HAL_OID)0x00000002)
#define		   NI_HAL_OID_ADD_ENET_CAM_ADDRESS		 			   ((NI_HAL_OID)0x00000003)
#define        NI_HAL_OID_CLEAR_ENET_CAM                       ((NI_HAL_OID)0x00000004)
#define        NI_HAL_OID_ENET_CAM_MASKREG                     ((NI_HAL_OID)0x00000005)


#define        NI_HAL_MODE_HAL80211_LOOPBACK_DISABLE           ((NI_HAL_MODE)0x00000000)
#define        NI_HAL_MODE_HAL80211_LOOPBACK_ENABLE            ((NI_HAL_MODE)0x00000001)
#define		   NI_HAL_MODE_SET_EMAC_CLEAR_CAM					   ((NI_HAL_MODE)0x00000002)  

/********************************************************************************
*    NI_HAL_OID and its values
*
********************************************************************************/

/*
 * NI_HAL_OID values
 *   OID values are 32 bit values laid out as follows:
 *
 *   3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
 *   1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
 *  +-+-----------------------------+-------------------------------+
 *  |M|          Unused             |               OID             |
 *  +-+-----------------------------+-------------------------------+
 *
 *  where
 *
 *      M - the multiple instances flag
 *        0 - Only one instance of the MAC OID can exist
 *        1 - Multiple instances of the MAC OID can exist
 *
 *      OID - the 16-bit OID value
 *
 */

typedef        UINT                                      HAL_80211_OID;
typedef        UINT                                      HAL_80211_MODE;

#define		   MIB_OID_80211_DTIMPERIOD       	         ((HAL_80211_OID)0x00000000)
#define		   MIB_OID_80211_BEACONINTERVAL    	         ((HAL_80211_OID)0x00000001)
#define		   MIB_OID_80211_WEPDEFAULTKEYID             ((HAL_80211_OID)0x00000002)
#define		   MIB_OID_80211_EXCLUDEUNENCRYPTED          ((HAL_80211_OID)0x00000003)
#define		   MIB_OID_80211_PRIVACYINVOKED       	      ((HAL_80211_OID)0x00000004)
#define		   MIB_OID_80211_DEFAULTKEY       	         ((HAL_80211_OID)0x00000005)
#define		   MIB_OID_80211_AUTHALGORITHM1              ((HAL_80211_OID)0x00000009)
#define		   MIB_OID_80211_AUTHALGORITHMENABLED1       ((HAL_80211_OID)0x0000000a)
#define		   MIB_OID_80211_TEMPTYPE                    ((HAL_80211_OID)0x0000000b)
#define		   MIB_OID_80211_PROTOCOLRSPTIMEOUT          ((HAL_80211_OID)0x0000000c)
#define		   MIB_OID_80211_SHORTRETRYLIMIT             ((HAL_80211_OID)0x0000000d)
#define		   MIB_OID_80211_LONGRETRYLIMIT              ((HAL_80211_OID)0x0000000e)
#define		   MIB_OID_80211_CFPMAXDURATION              ((HAL_80211_OID)0x0000000f)
#define		   MIB_OID_80211_CFPPERIOD                   ((HAL_80211_OID)0x00000010)
#define		   MIB_OID_80211_CFPMEDIUMOCCUPANCYLIMIT     ((HAL_80211_OID)0x00000011)
#define		   MIB_OID_80211_PHYTYPE                     ((HAL_80211_OID)0x00000012)
#define		   MIB_OID_80211_CURRENTCHANNEL              ((HAL_80211_OID)0x00000013)
#define		   MIB_OID_80211_CCAMODE                     ((HAL_80211_OID)0x00000014)
#define        MIB_OID_80211_FRAGTHRESH                  ((HAL_80211_OID)0x00000015)
#define        MIB_OID_80211_RTSTHRESH                   ((HAL_80211_OID)0x00000016)
#define        MIB_OID_80211_SSID                        ((HAL_80211_OID)0x00000017)
#define        MIB_OID_80211_CHANNELNUMBER               ((HAL_80211_OID)0x00000018)
#define        MIB_OID_80211_BASICRATES                  ((HAL_80211_OID)0x00000019)
#define        MIB_OID_80211_SUPPORTEDRATES              ((HAL_80211_OID)0x0000001a)
#define        MIB_OID_80211_TXRATECONTROL               ((HAL_80211_OID)0x0000001b)

/*******************************************************************************
*    Data buffer linked list structure used by the CM MAC to pass data (Ethernet)
*    packets.
*/

// This is for use by the CM HAL (and the HAL Interface module) to handle PHS.
// In PHS, the packet header is suppressed, which would cause the packet data
// to be modified.  Because the packet may also be used at the same time by
// other HALs (for broadcast or multicast), and in some cases the client expects
// that it will not be modified, we need to avoid modifying it.
//
// This structure supports associating a new "pre-buffer" with the main buffer,
// which contains the modified header data, as well as offset and
// length information.
//
// The pre-buffer is allocated by the HAL Interface module (via an Info call
// to the CM HAL), and is freed by the CM HAL when transmit is complete.
//
//   <------ uiLen -------->
//  +-----------------------+
//  | 1 2 3 4 5 6 7 8 9 ... |
//  +-----------------------+
//    s   s   s   s                 s = Byte to be suppressed.
//
// Here is what the packet would look like if s/w suppressed in-place (note
// the gap between the unsuppressed portion of the header and the remainder of
// the packet):
//
//  +-----------------------+
//  | 2 4 6 x x x x 8 9 ... |
//  +-----------------------+
//
// Here is how the pre-buffer is used to avoid modifying the packet:
//
//  +-------+  +-----------------------+
//  | 2 4 6 |  | 1 2 3 4 5 6 7 8 9 ... |
//  +-------+  +-----------------------+
//  ^          ^               ^
//  |          |               |
//  |          +-pvStart       mainBufferOffset = 7
//  pPreBuffer                 remainder length = uiLen - 7
//  numberOfBytes = 3
//
// ------------------------------
// Here's another example, where all bytes of the header are being suppressed;
// it shows that the pre-buffer can be empty/NULL, and only the main buffer
// offset used:
//
//   <------ uiLen -------->
//  +-----------------------+
//  | 1 2 3 4 5 6 7 8 9 ... |
//  +-----------------------+
//    s s s s s s s                 s = Byte to be suppressed.
//
//             +-----------------------+
//             | 1 2 3 4 5 6 7 8 9 ... |
//  NULL       +-----------------------+
//  ^          ^               ^
//  |          |               |
//  |          +-pvStart       mainBufferOffset = 7
//  pPreBuffer                 remainder length = uiLen - 7
//  numberOfBytes = 0
//
// ------------------------------
// As a last example, it is possible for all bytes of the packet to be
// suppressed, except for a few header bytes; in this case, pre-buffer may be
// the only data to be transmitted:
//
//   <------ uiLen -------->
//  +-----------------------+
//  | 1 2 3 4 5 6 7 8 9 ... |
//  +-----------------------+
//    s   s   s   s s s sss         s = Byte to be suppressed.
//
//  +-------+  +-----------------------+
//  | 2 4 6 |  | 1 2 3 4 5 6 7 8 9 ... |
//  +-------+  +-----------------------+
//  ^          ^                       ^
//  |          |                       |
//  |          +-pvStart               mainBufferOffset = uiLen
//  pPreBuffer                         remainder length = 0
//  numberOfBytes = 3
//

////////////////////////////////////////////////////////////////////////////////
// Specific DOCSIS CM PHS operation/flags for SW and HW PHS
//

typedef struct _CM_HAL_PHS_INFO
{
    // SW PHS support only:
    void *pPreBuffer;           // Buffer containing the unsuppressed portion of the header.
    uint16 numberOfBytes;       // The number of bytes of data in the pre-buffer.

    uint16 mainBufferOffset;    // The offset into the main buffer where the remainder of the packet starts.
								// The remainder length can be calculated (uiLen = mainBufferOffset).
 
    // HW PHS support only:
	byte hwPhsIndex;	   	  	// if > 0 specifies the phsIndex, if HW PHS is supported, otherwise = 0

} CM_HAL_PHS_INFO;

typedef struct _DATA_BUFFER_ {

   PVOID                   pvStart;         // Pointer to the packet's data buffer.
   UINT                    uiLen;           // The number of bytes of data in the buffer.

   CM_HAL_PHS_INFO         cmHalPhsInfo;    // NOTE:  Only for use by CM HAL (for PHS support)

   UINT                    ipsecOffset;     // Used by IPSEC to hold the offset needed to free the data
   
   // For buffer chaining; this is no longer supported in most HALs, so it
   // should not be used.  We may remove this member in the future.
   struct _DATA_BUFFER_    *pNext;

} DATA_BUFFER, *PDATA_BUFFER;

/*******************************************************************************
*    DOCSIS Extended Header Packet structure.
*    (note: may contain multiple DOCSIS Ehdrs, uiLen represents total EHDR length)
*/
typedef struct _DOCSIS_EXTENDED_HEADER_ {
   PBYTE    pbStart;
   UINT     uiLen;
} DOCSIS_EXTENDED_HEADER, *PDOCSIS_EXTENDED_HEADER;

/*******************************************************************************
*    Ethernet packet structure.
*/
typedef struct _ETHERNET_PACKET_ {

   BcmHandle      hPacket;
   PDATA_BUFFER   pDataBufferHead;
   PDATA_BUFFER   pDataBufferTail;
   ULONG          ulCount;                // Packet usage counter
   USHORT         uiInterface;            // interface this packet came from
   USHORT         uiDestination;          // interface this packet is being sent to
   DOCSIS_EXTENDED_HEADER eHdr;           // DOCSIS Extended Header object (used for Downstream)
   struct _ETHERNET_PACKET_ *pNext;       // Reserved field for CM HAL

}    ETHERNET_PACKET, *PETHERNET_PACKET;


/*******************************************************************************
*    DOCSIS management message structure.
*/
typedef struct _MGMT_MSG_ {

   BcmHandle    hPacket;
   PVOID     pvPacketStart;
   UINT      uiPacketLen;
   UINT      uiInterface;                 // driver/port logical interface ID (assigned by MAC Layer)
   DOCSIS_EXTENDED_HEADER eHdr;           // DOCSIS Extended Header object (used for Downstream)
   struct _MGMT_MSG_  *pNext;             // Reserved field for CM HAL

}    MGMT_MSG, *PMGMT_MSG;


/*******************************************************************************
*    DOCSIS header passed down from CM MAC for each packet to be sent out.
*/
typedef   struct _DOCSIS_HDR_ {

   BYTE      bFC;                          /* Frame Control: Identifies MAC Header type */
   BYTE      bMacParm;                     /* Parameter field: Dependent on FC */
   USHORT    usLen;                        /* Length of the MAC frame - always set to 0x0000, */
                                           /* will be overwritten by hardware */
   PBYTE     pbEhdr;                       /* Pointer to Extended Header if present or NULL */
   USHORT    usHcs;                        /* Header Checksum - always set to 0x0000, */
                                           /* will be overwritten by hardware */
} DOCSIS_HDR, *PDOCSIS_HDR;

/*******************************************************************************
*    Security packet structure.
*/
//  defines for Security
#define SECURE_3DESKEY_LENGTH 24
#define SECURE_DESKEY_LENGTH   8
#define SECURE_IV_LENGTH       8
#define SECURE_AUTHKEY_LENGTH 64   
#define SECURE_AUTHCHK_LENGTH 20
#define SECURE_AUTH_ERROR     0xFFFF  	 
#define SECURE_INTERNAL_ERROR 0xFFFE  

typedef
VOID
(*PFN_CM_HAL_SECURITY_HANDLER) (uint16,uint8,uint16);


// This structure gets sent along with the CM_HAL_OID_SECURE_PROCESS_HEADER OID
// These values do not typically get changed a lot
typedef  struct _SECURE_PARAMS_HEADER {
   BYTE securetype;          // secure type
#define  SEC_ENCODE 0
#define  SEC_DECODE 1
   BYTE secureprocess;       // secure process                 
#define  SEC_CRYPT 0
#define  SEC_AUTH   1
#define  SEC_BOTH   2
   BYTE secureoptions;       // Options Bit Mapped
#define  SECURE_AUTHCOPY 1   // Enable coping source data when Authentication Only   
   BYTE securecryptalg;      // crypto alogorim
#define  SEC_DES   0
#define  SEC_3DES 1 
   BYTE securecryptmode;     // crypto mode
#define  SEC_ECB 0
#define  SEC_CBC 1           
   BYTE secureauthalg;       // Authentication algorithm
#define  SEC_HMAC  0
#define  SEC_DIGEST 1           
  BYTE  secureauthmode;      // Authentication mode
#define  SEC_SHA  0
#define  SEC_MD5 1           
  BYTE  secureauthverlen;    // Authentication Verification length
  uint16 securecrypthlen;    // crypt header length
  uint16 securecrypttlen;    // crypt trailer length
  uint16 secureauthhlen;     // Authentication header length
  uint16 secureauthtlen;     // Authentication trailer length
  BYTE   securedeskey[SECURE_3DESKEY_LENGTH]; // DES or 3DES key  
  BYTE   secureauthkey[SECURE_AUTHKEY_LENGTH];   // Authentication key  
}SECURE_PARAMS_HEADER ; 

// This structure gets sent along with the CM_HAL_OID_SECURE_PROCESS_DATA OID
typedef  struct _SECURE_PARAMS_DATA {
  BYTE   secureseqnum;		     // sequence number
  BYTE   secureiv[SECURE_IV_LENGTH]; // Initialization vector when CBC defined
  BYTE   secureauthchk[ SECURE_AUTHCHK_LENGTH]; // Authentication check string 
  BYTE   securepackettype;        // packet type of source
#define SEC_PDATA     0          // pointer to source data only
#define SEC_PETHERNET 1          // pointer to Ethernet header
  uint16 securecryptalglen;		 // crypt algorithm length
  uint16 secureauthalglen;		 // Authentication algorithm length
  PFN_CM_HAL_SECURITY_HANDLER  securecallback;  // callback routine to use when data has been processed 
  uint16 securesourcelen;        // Length of source (only need if packetype = SEC_PDATA)
  uint16 secureresultlen;        // Length of result string 
  PVOID  securesource;           // Source string
  PVOID  secureresult;           // Result string
                                 // The result string must be large enough to hold the entire
                                 // result, and must be on a double word boundary. If you 
                                 // allocate using BcmAlloc, then this guarantees that the
                                 // memory is on a double word boundary. 
} SECURE_PARAMS_DATA ; 

/*******************************************************************************
*    Downstream Channel Parameters.
*/

#define DNSTREAM_MODULATION_UNKNOWN         1
#define DNSTREAM_MODULATION_OTHER           2
#define DNSTREAM_MODULATION_QAM64           3
#define DNSTREAM_MODULATION_QAM256          4
#define DNSTREAM_MODULATION_QAM16           5
#define DNSTREAM_MODULATION_ALL	 	        6

#define DNSTREAM_INTERLEAVE_UNKNOWN         1
#define DNSTREAM_INTERLEAVE_OTHER           2
#define DNSTREAM_INTERLEAVE_TAPS8_INCR16    3
#define DNSTREAM_INTERLEAVE_TAPS16_INCR8    4
#define DNSTREAM_INTERLEAVE_TAPS32_INCR4    5
#define DNSTREAM_INTERLEAVE_TAPS64_INCR2    6
#define DNSTREAM_INTERLEAVE_TAPS128_INCR1   7
#define DNSTREAM_INTERLEAVE_TAPS12_INCR17   8 /* Euro-docsis interleave */

/* DS configuration */
#define ANNEX_A 1
#define ANNEX_B 0
#define ANNEX_C 2

// use with CM_HAL_OID_DNSTREAM_CHANNEL_STATUS (Query Only)
typedef struct _DownstreamChannelStatus_ {
   BOOL bSyncAcquired;
   BOOL bFECLock;
   BOOL bQAMLock;
} DownstreamChannelStatus, *PDownstreamChannelStatus;

typedef struct _DNSTREAM_CHANNEL_PARAMS_ {

   ULONG     ulFreq;                       /* Center Frequency in Hertz */
   ULONG     ulWidth;                      /* Channel width in Hertz */
   ULONG     ulPower;                      /* Channel power level in tenth dBmV units */
   USHORT    usModulationType;             /* From the modulation types defined above */
   UCHAR     ucInterleave;                 /* From the interleave values defined above */
   UCHAR     ucAnnex;                      /* If NULL, default = Annex-B */
   ULONG     ulSymbolRate;                 /* If NULL, use MCNS symbol rate */
   BOOL      bSingleScan;                  /* If TRUE, will scan single freq pair */
                                           /* (1 HRC, 1 STANDARD) default is FALSE */
   DownstreamChannelStatus downstreamChannelStatus;  /* current Downstream channel status */
} DNSTREAM_CHANNEL_PARAMS, *PDNSTREAM_CHANNEL_PARAMS;

/*******************************************************************************
*    Change Downstream Frequency Structure
*    (used with CM_HAL_OID_DNSTREAM_CONFIG
*/

// Feature Descriptor specific definition for DOWNSTREAM_CHANNEL_SETTINGS
typedef enum
{
   ////////////////////////////////////////////////////////////////////////////////
   //  define the various descriptors for DOWNSTREAM_CHANNEL_SETTINGS  (bit wise)
   // 
   kDownsteamFrequencyChange           = 0x00000001,
   kDownsteamModulationChange          = 0x00000002,
   kDownsteamSymbolRateChange          = 0x00000004,
   kDownsteamInterleaverDepthChange    = 0x00000008,
   kDownsteamChannelIdChange           = 0x00000010,
   kDownsteamSyncSubstitutionIdChange  = 0x00000020
} DownstreamChannelDescriptor;

// Downstream Settings Params
typedef enum
{
   ////////////////////////////////////////////////////////////////////////////////
   //  define the various parameters for DOWNSTREAM_CHANNEL_SETTINGS
   // 
   kDownsteamModulation64Qam           = 0,
   kDownsteamModulation256Qam          = 1,

   kDownsteamSymbolRate_5_05Msym       = 0,  // 5.056941 Msym/sec
   kDownsteamSymbolRate_5_36Msym       = 1,  // 5.360537 Msym/sec
   kDownsteamSymbolRate_6_95Msym       = 2,  // 6.952 Msym/sec

   kDownsteamAcquireSyncMessage        = 0,
   kDownsteamDoNotAcquireSyncMessage   = 1

} DownstreamSettingsParams;

typedef struct _DOWNSTREAM_CHANNEL_SETTINGS_ {
   uint32   featureDescriptor;   // bit definition of feature to enable (DownstreamChannelDescriptor)
   uint32   frequency;
   byte     modulation;          // specified in DownstreamSettingsParams
   byte     symbolRate;          // specified in DownstreamSettingsParams
   byte     interleaverDepthI;
   byte     interleaverDepthJ;
   byte     channelId;
   byte     syncSubstitution;    // specified in DownstreamSettingsParams  
   PDNSTREAM_CHANNEL_PARAMS pDownstreamChannelParams;   // used for query case (pointer provided by caller)
} DOWNSTREAM_CHANNEL_SETTINGS, *PDOWNSTREAM_CHANNEL_SETTINGS;


/*******************************************************************************
*    UCD Burst Descriptor Structure
*/

typedef struct _BURST_DESC_ {

   BYTE        bIuc;
   BYTE        bModulationType;
   BYTE        bDiffEncodingOnOff;
   USHORT      usPreambleLen;
   USHORT      usPreambleValOffset;
   BYTE        bFecBytes;
   BYTE        bFecCodewordInfoBytes;
   USHORT      usScramblerSeed;
   BYTE        bMaxBurstSize;
   BYTE        bGuardTimeSize;
   BYTE        bLastCodewordLen;
   BYTE        bScramblerOnOff;

   // DOCSIS 2.0 extensions (Advanced Phy)
   BYTE		   rsInterleaverDepth;
   USHORT	   rsInterleaverBlockSize;
   BYTE		   preambleType;

} BURST_DESC, *PBURST_DESC;


/*******************************************************************************
*    UCD Structure
*/
												//Increased to 1536 bits to support DOCSIS 2.0 (Adv Phy)
#define	MAX_UCD_PREAMBLE_PATTERN_LEN			192 //128 
#define	MAX_UCD_BURST_DESCS						16

typedef struct _UCD_ {

   ULONG       ulUpstreamFreq;
   BYTE        bUpstreamChannelId;
   BYTE        bConfigChangeCount;
   BYTE        bMiniSlotSize;
   BYTE        bDnstreamChannelId;
   BYTE        bSymbolRate;
   BYTE        bPreamblePatternLen;
   BYTE        bBurstDescCount;
   BYTE        pbPreamblePattern[MAX_UCD_PREAMBLE_PATTERN_LEN];
   BURST_DESC  pBurstDesc[MAX_UCD_BURST_DESCS];

} UCD, *PUCD;


/*******************************************************************************
*    Ranging Adjustments Structure
*/

typedef struct _COMPLEX_COEF_ {

   SHORT           sReal;
   SHORT           sImaginary;

} COMPLEX_COEF, *PCOMPLEX_COEF;

typedef struct _TRANSMIT_EQ_COEF_ {

   UCHAR           ucTapsPerSymbol;
   UCHAR           ucNumFwdTaps;
   UCHAR           ucNumRevTaps;
   UCHAR           ucMainTapLocation;
   UCHAR           ucCoefOperation;         // PR1590 - DOCSIS 2.0 support
   PCOMPLEX_COEF   pComplexCoefFwdTaps;
   PCOMPLEX_COEF   pComplexCoefRevTaps;

} TRANSMIT_EQ_COEF, *PTRANSMIT_EQ_COEF;

// PR1590 - DOCSIS 2.0 support
#define INIT_COEFS 0
#define CONVOLVE_COEFS 1

typedef struct _RNG_ADJUST_ {

   USHORT               usSid;
   UCHAR                ucRangingStatus;
   LONG                 lTiming;
   SHORT                sOffsetFreq;
   char                 cPwrLevel;
   UCHAR                ucFineTiming;       // PR1590 - DOCSIS 2.0 support
   PTRANSMIT_EQ_COEF    pTransmitEqCoef;

} RNG_ADJUST, *PRNG_ADJUST;


////////////////////////////////////////////////////////////////////////////////
// Descriptor specific definition for the TEK PARAMS interface
typedef enum
{
   ////////////////////////////////////////////////////////////////////////////////
   //  define the various descriptors for the TEK Params interface
   // 
   kNoKey         = 0x00000001,
   kPrimarySaid   = 0x00000002,
   kUnicastSaid   = 0x00000004,
   kMulticastSaid = 0x00000005
} TekParamsDescriptor;

/*******************************************************************************
*    TEK Parameters Structure
*/

#define DES_KEY_LEN         8
#define DES_CBC_IV_LEN      8

typedef struct _TEK_PARAMS_ {

   BYTE     pbDesKey[DES_KEY_LEN];
   BYTE     pbDesCbcIv[DES_CBC_IV_LEN];
   BYTE     bKeySeqNum;
   ULONG    ulLifeTime;
   BcmHandle   hUsServiceFlow;   // BcmHandle to Upstream service flow for TEK_PARAMS (legacy)
   long     tekParamsDescriptor; // Descriptor associated with this TEK
   uint16   sid;                 // SID associated with this TEK 
} TEK_PARAMS, *PTEK_PARAMS;


////////////////////////////////////////////////////////////////////////////////
// Upstream Service Flow Attributes
typedef enum
{
   ////////////////////////////////////////////////////////////////////////////////
   // These are the bits that are in the Request/Transmission Policy for upstream
   // Service Flows.
    kDisableBroadcastRequestBit        = 0x00000001,
    kDisableMulticastRequestBit        = 0x00000002,
    kDisableRequestDataForRequestBit   = 0x00000004,
    kDisableRequestDataForDataBit      = 0x00000008,
    kDisablePiggybackRequestsBit       = 0x00000010,
    kEnableConcatenateDataBit          = 0x00000020,
    kEnableFragmentDataBit             = 0x00000040,

    // General Service Flow Control  
    // these policies are reserved for internal CM HAL use only (caller MUST use MODE API instead!)
    kUsServiceFlowActive               = 0x00000080,
    kUsServiceFlowInhibit              = 0x00000100,

    // Priority Service Flow (PSF) policy/control
    kPriorityServiceFlow                  = 0x00010000,
    kReactivateParentServiceFlow          = 0x00020000,
    kPriorityServiceFlowGlobalAssociated  = 0x00040000,

    // Internal CM HAL use only
    kPriorityServiceFlowReady             = 0x00080000,
    kHwPhsRule                            = 0x00100000,

    // Additional flow/scheduling policies:
    kEnableUGSDataBit                     = 0x40000000

} UpstreamServiceFlowAttributes;

////////////////////////////////////////////////////////////////////////////////
//  CM HAL Upstream Service Flow structure (CM MAC API for managing service flows)
//
typedef struct _UPSTREAM_SERVICE_FLOW {
   BcmHandle   hUsServiceFlow;
   BcmHandle   hParentUsServiceFlow;      // for priority service flow (PSF)
   uint16   sid;
   uint16   grpSid;
   uint16   usMaxQueueDepth;
   uint32   attributes;   // from UpstreamServiceFlowAttributes
   uint32   reserved;
   uint16   usMaximumConcatenatedBurst;
   byte     ucCCThreshold;	  	// minimum concat packet threshold
   byte     ucCCMaxThreshold;	// maximum concat packet threshold
   uint16   ccThresholdPercentOfMax; // Percentage of Maximum burst required to concatenate the frame (from non-vol)
} UPSTREAM_SERVICE_FLOW, *PUPSTREAM_SERVICE_FLOW;


#define DATA_CACHE_INVALIDATE_LENGTH 92               // Number of bytes that will be cleared 
                                                      // from cache to allow new received data to
                                                      // be cached.

/*******************************************************************************
*    CM_HAL_CHARACTERISTICS and all its prototypes.
*
*******************************************************************************/

#define MAJOR_CM_HAL_VERSION        0x01
#define MINOR_CM_HAL_VERSION        0x00

// SID Priority definitions (for Upstream transmission).
// Priority range is from 0..(MAX_SIDS - 1). Higher values imply higher priority.

#define CM_HAL_SID_PRIORITY_LOWEST          0

// Upstream Transmit Flag definitions

#define CM_HAL_UPSTREAM_PACKET_DEFAULT          0x00   // default US Best-Effort packet
#define CM_HAL_UPSTREAM_PACKET_START_CONCAT     0x80   // packet is start of a concatenation sequence
#define CM_HAL_UPSTREAM_PACKET_CONCAT           0x40   // packet is part of a concatenation sequence
#define CM_HAL_UPSTREAM_PACKET_CONCAT_GRP_MAX   0x20   // CC group contains maximum packets for current US parms
#define CM_HAL_UPSTREAM_PACKET_END_CONCAT       0x10   // packet is end of a concatenation sequence
#define CM_HAL_UPSTREAM_PACKET_CBR              0x08   // CBR (constant bit rate packet)


typedef
CM_STATUS
(*PFN_CM_HAL_INIT_HANDLER) (
                           VOID );

typedef
CM_STATUS
(*PFN_CM_HAL_SCAN_DNSTREAM_CHANNEL_HANDLER) (
                                            PDNSTREAM_CHANNEL_PARAMS pDnstreamChannelParams );

typedef
CM_STATUS
(*PFN_CM_HAL_SCAN_DNSTREAM_FREQUENCY_HANDLER) (
                                            PDNSTREAM_CHANNEL_PARAMS pDnstreamChannelParams );

typedef
CM_STATUS
(*PFN_CM_HAL_SET_INFO_HANDLER) (
                               CM_HAL_OID cmHalOid,
                               ULONG      ulParam,
                               PVOID      pvInfoBuffer,
                               ULONG      ulInfoBufferLen );

typedef
CM_STATUS
(*PFN_CM_HAL_QUERY_INFO_HANDLER) (
                                 CM_HAL_OID CmHalOid,
                                 ULONG      ulParam,
                                 PVOID      pvInfoBuffer,
                                 PULONG     pulInfoBufferLen );

typedef
CM_STATUS
(*PFN_CM_HAL_SET_MODE_HANDLER) (
                               CM_HAL_MODE CmHalMode,
                               ULONG       ulParam,
                               BOOLEAN     bflag );

typedef
CM_STATUS
(*PFN_CM_HAL_QUERY_MODE_HANDLER) (
                                 CM_HAL_MODE CmHalMode,
                                 ULONG       ulParam,
                                 PBOOLEAN    pbflag );

typedef
VOID
(*PFN_CM_HAL_RELEASE_MGMT_MSG_HANDLER) (
                                       PMGMT_MSG pMgmtMsg );

typedef
VOID (*PFN_CM_HAL_RELEASE_DATA_PACKET_HANDLER) (
                                               PETHERNET_PACKET pEthernetPacket );

typedef
CM_STATUS
(*PFN_CM_HAL_SEND_MGMT_MSG_HANDLER) (
                                    PDOCSIS_HDR pDocsisHdrMgmtMsg,
                                    BcmHandle      hUsServiceFlow,
                                    PMGMT_MSG   pMgmtMsg );

typedef
CM_STATUS
(*PFN_CM_HAL_SEND_DATA_PACKET_HANDLER) (
                                       PDOCSIS_HDR      pDocsisHdrDataPacket,
                                       PETHERNET_PACKET pEthernetPacket,
                                       BcmHandle           hUsServiceFlow,
                                       UCHAR            ucTxFlags );

typedef   struct _CM_HAL_CHARACTERISTICS_ {

   USHORT                                       usMajorCmMacVersion;
   USHORT                                       usMinorCmMacVersion;
   PFN_CM_HAL_INIT_HANDLER                      InitHandler;
   PFN_CM_HAL_SCAN_DNSTREAM_CHANNEL_HANDLER     ScanDnstreamChannelHandler;
   PFN_CM_HAL_SCAN_DNSTREAM_FREQUENCY_HANDLER   ScanDnstreamFrequencyHandler;
   PFN_CM_HAL_SET_INFO_HANDLER                  SetInfoHandler;
   PFN_CM_HAL_QUERY_INFO_HANDLER                QueryInfoHandler;
   PFN_CM_HAL_SET_MODE_HANDLER                  SetModeHandler;
   PFN_CM_HAL_QUERY_MODE_HANDLER                QueryModeHandler;
   PFN_CM_HAL_RELEASE_MGMT_MSG_HANDLER          ReleaseMgmtMsgHandler;
   PFN_CM_HAL_RELEASE_DATA_PACKET_HANDLER       ReleaseDataPacketHandler;
   PFN_CM_HAL_SEND_MGMT_MSG_HANDLER             SendMgmtMsgHandler;
   PFN_CM_HAL_SEND_DATA_PACKET_HANDLER          SendDataPacketHandler;

} CM_HAL_CHARACTERISTICS, *PCM_HAL_CHARACTERISTICS;


/*******************************************************************************
*    CM_MAC_CHARACTERISTICS and all its prototypes.
*
********************************************************************************/

#define MAJOR_CM_MAC_VERSION        0x01
#define MINOR_CM_MAC_VERSION        0x00

typedef
CM_STATUS
(*PFN_CM_MAC_REGISTER_HAL_HANDLER) (
                                   PCM_HAL_CHARACTERISTICS pCmHalCharacteristics, BcmHandle hMac );

typedef
VOID
(*PFN_CM_MAC_INDICATE_MGMT_MSG_HANDLER) (
                                        PMGMT_MSG pMgmtMsg, BcmHandle hMac );

typedef
VOID
(*PFN_CM_MAC_INDICATE_DATA_PACKET_HANDLER) (
                                           PETHERNET_PACKET pEthernetPacket, BcmHandle hMac );

typedef
VOID
(*PFN_CM_MAC_DATA_PACKET_SEND_COMPLETE_HANDLER) (
                                                PETHERNET_PACKET pEthernetPacket, BcmHandle hMac );

typedef
VOID
(*PFN_CM_MAC_MGMT_MSG_SEND_COMPLETE_HANDLER) (
                                             PMGMT_MSG pMgmtMsg, BcmHandle hMac );

typedef
VOID
(*PFN_CM_MAC_SEND_RESOURCES_AVAILABLE_HANDLER) (
                                               BcmHandle hMac );

typedef
VOID
(*PFN_CM_MAC_INDICATE_STATUS_HANDLER) (
                                      CM_MAC_OID CmMacOid,
                                      ULONG      ulParam,
                                      PVOID      pvStatusBuffer,
                                      ULONG      ulStatusBufferLen,
                                      BcmHandle     hMac );

typedef
CM_STATUS
(*PFN_CM_MAC_ALLOCATE_DATA_BUFFER_MEMORY) (
                                          PVOID     *ppvAddr,
                                          UINT      uiLen,
                                          BcmHandle    hMac );


typedef struct _CM_MAC_CHARACTERISTICS {
   USHORT                                       usMajorVersion;
   USHORT                                       usMinorVersion;
   UINT                                         uiInterfaceNumber;
   PFN_CM_MAC_REGISTER_HAL_HANDLER              RegisterHalHandler;
   PFN_CM_MAC_INDICATE_MGMT_MSG_HANDLER         IndicateMgmtMsgHandler;
   PFN_CM_MAC_INDICATE_DATA_PACKET_HANDLER      IndicateDataPacketHandler;
   PFN_CM_MAC_DATA_PACKET_SEND_COMPLETE_HANDLER DataPacketSendCompleteHandler;
   PFN_CM_MAC_MGMT_MSG_SEND_COMPLETE_HANDLER    MgmtMsgSendCompleteHandler;
   PFN_CM_MAC_SEND_RESOURCES_AVAILABLE_HANDLER  SendResourcesAvailableHandler;
   PFN_CM_MAC_INDICATE_STATUS_HANDLER           IndicateStatusHandler;
   PFN_CM_MAC_ALLOCATE_DATA_BUFFER_MEMORY       AllocateDataBufferMemoryHandler;
   PVOID                                        pvCableModemParms;
   BcmHandle                                       hMac;
   BcmHandle                                       hHalThreadPriority;
} CM_MAC_CHARACTERISTICS, *PCM_MAC_CHARACTERISTICS;


typedef struct _CableModemParms {
   unsigned char  UpstreamAmp;
   unsigned long  InputFreqToPhy;
   unsigned long  DSReferenceFrequency;
   unsigned long  USReferenceFrequency;
   unsigned char  Tuner;
   long           boardRevision;
   unsigned short annexMode;
   float		  equalizerThreshold;
}CableModemParms;

////////////////////////////////////////////////////////////////////////////////
// Board Revision Constants
typedef enum
{
   kGenericUnknown                  = 0,

   /////////////////////////////////////////////////////////////////////////////
   // US Boards:
   kBCM93350A_V2_US                 = 1,
   kBCM93350A_V3_US                 = 2,     // With Maxim Upstream Amp
   kBCM93350C_V0_US                 = 3,
   kBCM93350i_V1a_US                = 4,     // With Enet Phy
   kBCM93350i_V1b_US                = 5,     // Without Enet Phy
   kBCM93350V_V1_US                 = 6,
   kBCM93352V_V0_US                 = 7,

    /////////////////////////////////////////////////////////////////////////////
    // Euro Boards:
   kBCM93350A_V2_EURO               = 8,
   kBCM93350A_V3_EURO               = 9,     // With Maxim Upstream Amp
   kBCM93350C_V0_EURO               = 10,
   kBCM93350i_V1a_EURO              = 11,     // With Enet Phy
   kBCM93350i_V1b_EURO              = 12,     // Without Enet Phy
   kBCM93350V_V1_EURO               = 13,
   kBCM93352V_V0_EURO               = 14,

   /////////////////////////////////////////////////////////////////////////////
   // The values for the previous boards were packed together, so we can't
   // group new values according to US/EURO.
   kBCM93350C_V2_US                 = 15,   // CMOS tuner, MAXIM amp.  The LEDS
   kBCM93350C_V2_EURO               = 16,   // have negative polarity as compared
                                            // to the V0 boards.
   kBCM93345I                       = 17,
   kBCM93345                        = 18,
   kBCM93351						= 19,
   kBCM93360						= 20
} BoardRevision;

////////////////////////////////////////////////////////////////////////////////
//  The adapter structure that control the operational of Cable Modem hardware.
//
typedef struct CM_ADAPTER {
   CM_MAC_CHARACTERISTICS  CmMacCharacteristics;

   DWORD   MacInterruptId;
   BcmHandle  hMacIST;
   BcmHandle  hMacIntEvent;
   UCHAR * pucMacSramBaseAddress;
   UCHAR * pucMacBaseAddress;
   UCHAR   bCmHalSendState;
   INT  uiInterfaceNumber;
   UCHAR MacAddr[6];
   ULONG IPAddr;

   CableModemParms cableModemParms;

}   CM_ADAPTER, *PCM_ADAPTER;

//////////////////////////////////////////////////////////////////////////////
//  The adapter structure that controls the operational of network interface
//  hardware.
//
typedef struct NI_ADAPTER 
{

   CM_MAC_CHARACTERISTICS  	CmMacCharacteristics;
   INT  					uiInterfaceNumber;

}   NI_ADAPTER, *PNI_ADAPTER;

//////////////////////////////////////////////////////////////////////////////
// Structure used by ethernet driver for init specific functions.           //
typedef struct ENETNIINITSTRUCT
{
	
	long	   boardRevision;
	BOOL	   autonegEnable;
	BOOL	   fullDuplex;
	UINT16	linkSpeed;
   BOOL     EnableEmacCam;
   BOOL     EnableCamAsReject;

} EnetNiInitStruct;

//////////////////////////////////////////////////////////////////////////////
// Structure used by ethernet driver for init specific functions.           //
typedef struct USBNIINITSTRUCT
{
	
	long	boardRevision;
	UCHAR	macAddress[6];
	UINT16	deviceId;
	UINT16	vendorId;

} UsbNiInitStruct;

//////////////////////////////////////////////////////////////////////////////
// Structure used by Bluetooth driver for init specific functions.           //
typedef struct BTNIINITSTRUCT
{
	
	long	boardRevision;
	UCHAR	LocalMacAddress[6];
	UCHAR	RemoteMacAddress[6];
   BOOL  MasterEnable;

} BtNiInitStruct;

//////////////////////////////////////////////////////////////////////////////
// Structure used by Bluetooth driver for init specific functions.           //
typedef struct HAL80211NIINITSTRUCT
{
/*
#define kSsidSize        33
#define kSecretKeySize   13
#define kStationListSize 5
#define kKeyTableRows    4
#define kKeySize         5
*/
	long	boardRevision;
	UCHAR	LocalMacAddress[6];
    uint8 uc80211SSID[33];	// SSID
    uint16 us80211ChannelNumber;	// channel number
	uint16 us80211BasicRates;
	uint16 us80211SupportedRates;
	uint16 us80211TxRateControl;
    uint16 us80211EncryptionMode;
	uint16 us80211Authentication;
    uint8 puc80211KeyTable[4][5];	// 4 rows, 5-byte keys
    uint16 us80211DefaultKey;		// which key is default
    uint8 puc80211SecretKey[13];	// 13-byte key
	uint16 us80211BeaconInterval;
	uint16 us80211FragThreshold;
	uint16 us80211RTSThreshold;
} Hal80211NiInitStruct;

//////////////////////////////////////////////////////////////////////////////
// Structure used by hpna driver for init specific functions.           //
typedef struct HPNANIINITSTRUCT
{
	
	long	boardRevision;
	int		debugMsgLevel;
} HpnaNiInitStruct;

////////////////////////////////////////////////////////////////////////////////
//  Upstream Service Flow structure used by HAL (internally) to manage service flows 
//
typedef struct _UsServiceFlow {
   byte     usQueue;   
   uint16   sid;
   uint16   grpSid;
   uint32   attributes;    // from UpstreamServiceFlowAttributes
   uint16   usMaxQueueDepth;
   uint16   usCurrentQueueDepth;           // queue depth (includes Software list and Hardware ring)
   uint16   usCurrentHardwareQueueDepth;   // actual queue depth in the hardware DMA ring
   uint16   usTruncatedHardwareQueueCount; // Number of packets in "truncated" queue for this flow (use with inhibit)
   uint16   usMaximumConcatenatedBurst;
   byte     ucCCThreshold;      // minimum concat packet threshold
   byte     ucCCMaxThreshold;	// maximum concat packet threshold

   // Service Flow de-InHibit support (applies to Flows that are inhibited - to restore to normal operation)
   PVOID    pInHibitPktDesc;              // Packet Descriptor to restore (if not NULL)

   // Priority Service Flow (PSF) support
   BcmHandle   hCompanionUsServiceFlow;      // handle of Parent if this is a PSF, or handle of PSF if this is a parent

   // diagnostics
   uint32	pktsReclaimed;					// packets reclaimed (transmitted)
   uint32	pktsDiscarded;					// packets discarded
   uint32	ccBackoutCnt;					// concatenated packets backed out
   uint32   ccTotalCnt;  	   				// concatenated packets (including backed out count)
   
   uint16   ccThresholdPercentOfMax;		// Percentage of Maximum burst required to concatenate the frame (from non-vol)
   uint16   minConcatBytes;					// minimum number of bytes for flow required to concatenate (computed from ccThresholdPercentOfMax)
   uint16   currentConcatQueuedFrameCount;  // current queued concat packet groups (frames)
   uint16   maxConcurrentConcatGroups;		// specifies the threshold for max CC groups at given time in the HW queue

   byte 	phsMaskBitsSet;					// for HW PHS support to allow the actual tx length to be re-computed (kHwPhsRule attribute SFs only)

   struct _UsServiceFlow * pNext;
}UsServiceFlow, *PUsServiceFlow;

/*******************************************************************************
*    CM HAL Driver Entry Function
*/

typedef
CM_STATUS
(*PFN_CM_HAL_DRIVER_ENTRY) (
                           PCM_MAC_CHARACTERISTICS pCmMacCharacteristics );

////////////////////////////////////////////////////////////////////////////////
//  Struct used by upstream test info.
//
typedef struct _US_TEST_STRUCT_ {
   float Rate;
   float Freq;
   float Power;
   char QAM_mode[16];
} US_TEST_STRUCT;

typedef struct _US_SWEEP_STRUCT_ {
   float StartFreqMHz;
   float EndFreqMHz;
   float StepSizeMHz;
   float Power;
   float Duration;
} US_SWEEP_STRUCT;

typedef struct _DS_TEST_STRUCT_ {
   float FreqMHz;
   float ifFreq;
   int NumberOfTries;
   unsigned short AnnexMode;
   unsigned short QAM; // 64/256 0=unspecified
   unsigned long TunerAGC;
   unsigned long IfAGC;
   unsigned char QAMLock;
   unsigned char FECLock;
} DS_TEST_STRUCT;


/* Struct used by CM_HAL_OID_READ_LDAIx_REGISTERS.  Note that both the LDAIT and
   LDAII registers are 24 bits in size; only the lower 24 bits of the values in
   this struct will have real data.  The upper byte is filled with 0's. 
   The LDAIF register was added to provide more precise values.  It has the
   same characteristics as the others. */
typedef struct _LDAIx_VALUES
{
    uint32 ldait;
    uint32 ldaii;

    uint32 ldaif;

    /* More values can be added if we decide that we need other registers as
       part of the calibration procedure... */
} LdaixValues;



////////////////////////////////////////////////////////////////////////////////
// Diagnostic/test Objects for displaying CM HAL Internal Information
// 
typedef enum {
   kCmHalDescriptorInfo		= 0x00000001,
   kCmHalQueueInfo			= 0x00000002
}CmHalObjectType;

typedef enum {
   kCmHalListingInfo		= 0x00000001,
   kCmHalSettingsInfo		= 0x00000002,
   kCmHalCountersInfo		= 0x00000004
}CmHalObjectProperty;

typedef struct _CmHalInternalInfo_  {
   uint32 objectType;  // type of object from enum CmHalObjectType
   uint32 property;    // property of object from CmHalObjectProperty
}CmHalInternalInfo;


#ifdef	PACKET_PROBE
////////////////////////////////////////////////////////////////////////////////
// Diagnostic/test Objects for system profiling
//
#define	PACKET_PROBE_ADDRESS	0xba000000			// EBI location of packet probe:
typedef struct _PacketProbe {
   uint32	address;								// 	0xba000000
   uint16	length;									//	0xba000004
   byte		signal;									//	0xba000006
#define	PACKET_PROBE_SIGNAL_UPSTREAM_CAPTURE_ON		0x01
#define	PACKET_PROBE_SIGNAL_UPSTREAM_CAPTURE_OFF	0x02
#define	PACKET_PROBE_SIGNAL_DOWNSTREAM_CAPTURE_ON  	0x04
#define	PACKET_PROBE_SIGNAL_DOWNSTREAM_CAPTURE_OFF	0x08
   byte		unused0;
}PacketProbe;
#endif

#if defined __cplusplus
}
#endif

#endif    /* CBLMODEM_H */

