/***************************************************************************
*    Copyright 2000  Broadcom Corporation
*    All Rights Reserved
*    No portions of this material may be reproduced in any form without the
*    written permission of:
*             Broadcom Corporation
*             16251 Laguna Canyon Road
*             Irvine, California  92618
*    All information contained in this document is Broadcom Corporation
*    company private, proprietary, and trade secret.
*
****************************************************************************
*
*    Filename: bcmosCfg.h
*    Creation Date: 10 Oct 2000 (v0.00)
*    VSS Info:
*        $Revision: 1.1 $
*        $Date: 2002/08/21 17:54:19 $
*
****************************************************************************
*    Description:
*
*     BCMOS default build options.  To override defaults in
*     bcmosCfg.h make a file called bcmosCfgCustom.h in your build
*     directory.
*
****************************************************************************/

#ifndef BCMOS_CFG_H
#define BCMOS_CFG_H

#ifdef __cplusplus
extern "C" {
#endif

#include <bcmos.h>

/*
*****************************************************************************
** CONSTANT DECLARATIONS
*****************************************************************************
*/


/*
*****************************************************************************
** Domain name service (DNS)
*****************************************************************************
*/
#define OS_CFG_DNS_TASK_PRIORITY    OS_TASK_MANAGEMENT


/*
*****************************************************************************
** File descriptor sets
*****************************************************************************
*/
#define OS_CFG_MAX_FD_SETS          16


/*
*****************************************************************************
** Tasks
*****************************************************************************
*/
#define OS_CFG_MAX_TASKS            16



/*
*****************************************************************************
** Application OS inheritance of BCMOS functions
*****************************************************************************
*/

/* Application code will have certain BCMOS functions overridden (listed below).
 * Attempts to call BCMOS functions that are overridden will  actually call the
 * APPOS equivalent functions. */

/* BCMOS functions should only be overridden for application code. No overriding
 * should be performed for APPOS or BCMOS (since the APPOS needs to call the REAL
 * BCMOS functions and not the overridden equivalents). */
#ifndef BUILDING_OS

   /* Rename overridden OS functions here */
   #define bcmOsSemTake             appOsSemTake
   #define bcmOsSleep               appOsSleep
   #define bcmOsQueueReceive        appOsQueueReceive
   #define bcmOsTaskSuspend         appOsTaskSuspend
   #define bcmOsTaskResume          appOsTaskResume
   #define bcmOsTaskDelete          appOsTaskDelete
   #define bcmOsSelect              appOsSelect
   #define bcmOsSocketReceiveFrom   appOsSocketReceiveFrom
   #define bcmOsEventReceive        appOsEventReceive
   #define bcmOsMutexLock           appOsMutexLock
   #define bcmOsTftpReadFile        appOsTftpReadFile

   /* BCMOS functions which are private, and must not be invoked by application code.
    * Rename private functions to undefined symbols that will cause link errors. */
//   #define bcmOsTaskCreate          bcmOsTaskCreate_IS_PRIVATE_USE_APPOS_VERSION
//   #define bcmOsSocketCloseShared   bcmOsSocketCloseShared_IS_PRIVATE_USE_APPOS_VERSION

#endif



/*
*****************************************************************************
** CUSTOM CONFIGURATION - This must be included after all default
** build configuration options. Custom include files should undef default
** config options and then re-define them.
*****************************************************************************
*/
#include <bcmosCfgCustom.h>


#ifdef __cplusplus
    }
#endif

#endif  /* BCMOS_CFG_H  */


