//**************************************************************************
//
//    Copyright 1999  Broadcom Corporation
//    All Rights Reserved
//    No portions of this material may be reproduced in any form without the
//    written permission of:
//             Broadcom Corporation
//             16251 Laguna Canyon Road
//             Irvine, California  92618
//    All information contained in this document is Broadcom Corporation
//    company private, proprietary, and trade secret.
//
//
//
//**************************************************************************
//    Filename: bcmos.h
//    Author:   Dannie Gay
//    Creation Date: 21-feb-99
//
//**************************************************************************
//    Description:
//
//	Header file for os specific definitions/includes
//		
//**************************************************************************
//    Revision History:
//
//		0.1  initial revision
//
//**************************************************************************

#ifndef bcmos_h
#define bcmos_h

#if __cplusplus
extern "C" {
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//#include "taskLib.h"
//#include "vmexLib.h"
//#include "semLib.h"
//#include "msgQLib.h"
#include <cyg/kernel/kapi.h>

#include "bsp.h"
///#include "ecboard.h"
#include "math.h"
//#include "vxWorks.h"
//#include <intLib.h>
//#include <sys/socket.h>
//#include <sockLib.h>
#include <sys/types.h>
#include <netinet/if_bcm.h>

#include "zone.h"
#include "bcmtypes.h"
//#include "cblmodem.h"
///#include "board.h"
#include <sys/param.h>
#include "timerUtils.h"

/* JRM - These setsockopt flags are used in the socket options
  and must be able to fit in a (short) - the standard flags are defined in socket.h
  These flags allow us to call setsocketopt() and specify an interface to send 
  directly to without going through the routing table. 
*/                     
#define BCM0_IFFLAG    BCM_NETIF_1
#define BCM1_IFFLAG    BCM_NETIF_2
#define BCM2_IFFLAG    BCM_NETIF_3
#define BCM3_IFFLAG    BCM_NETIF_4
#define BCM4_IFFLAG    BCM_NETIF_5
#define BCM5_IFFLAG    BCM_NETIF_6
#define BCM6_IFFLAG    BCM_NETIF_7
#define BCM7_IFFLAG    BCM_NETIF_8

#define	EBADS	0x5009		//in EventLog.cpp

#define HAL_THREAD_PRIORITY_TABLE_LENGTH  8
#define HAL_THREAD_PRIORITY               31
#define HAL_THREAD_STACK_SIZE             4500
#define HAL_THREAD_RX_STACK_SIZE          4500
#define HAL_THREAD_TX_STACK_SIZE          4500

// Note: Define out BSP logging if we are compiling to reduce the image size.
#if (BCM_REDUCED_IMAGE_SIZE)
#define DEBUGMSG(z, s, w...)
#define LogRawBuffer(s, p, b, w) 
#define HalPrint(s...)              
#define	HalDebugPrint(zn, s...)     
#define MacPrint(x...)	            
#else
void LogRawBuffer (char * str, uint8 *pBuffer, int BufLen, BOOL wacky);
int  DEBUGMSG(int zone, char *FmtStr, ...);
#define HalPrint(s...)              printf(s)
#define	HalDebugPrint(zn, s...)     DEBUGMSG(zn, s )
#define MacPrint(x...)	            printf(x)
#endif

int Print(char *,...);
void Sleep(int x);


typedef enum
{
   BCM_STATUS_OK = 0,   /**< Sucessful    */
   BCM_STATUS_TIMEOUT,  /**< Timeout      */
   BCM_STATUS_ERR,      /**< Failed       */

   /** For socket operations, indicates that a send or recv would block on
       a non-blocking socket.                                              */
   BCM_STATUS_WOULD_BLOCK,

   BCM_STATUS_RESET

} BCM_STATUS;

//--------------------------------------------------------------------------
// Virt/Physical Macros
//--------------------------------------------------------------------------
#define  CacheToNonCacheVirtAddress    K0_TO_K1
#define  NonCacheToCacheVirtAddress    K1_TO_K0
#define  CacheVirtToPhyAddress         K0_TO_PHYS
#define  NonCacheVirtToPhyAddress      K1_TO_PHYS
#define  PhyToNonCacheVirtAddress      PHYS_TO_K1
#define  PhyToCacheVirtAddress         PHYS_TO_K0
// the following are for compatibility with MMU based systems (i.e Linux)
#define  PhyToNonCacheVirtAddressMap(address, regionSize)	PhyToNonCacheVirtAddress(address)
#define  PhyToCacheVirtAddressMap(address, regionSize)      PhyToCacheVirtAddress(address)

// alignment macro
#define 	BCM_ALIGN(addr, boundary) ((addr + boundary - 1) & ~(boundary - 1))
#define  	BCM_ALIGN_ADJUST( boundary, objSize ) ((boundary/objSize) + ((boundary % objSize) ?  1 : 0))

#ifdef SETTOP_BUILD
uint32 BcmTimeMuLock(BcmHandle mu, uint32 msecs );
uint32 BcmTimeSemLock(BcmHandle sem, uint32 msecs );
#endif



int    BcmosInit(void);
bool   BcmMemProbeAddress(uint32 probeAddr, int accessSize);
uint32 BcmMuCreate(BcmHandle * mu);
uint32 BcmMuLock(BcmHandle mu);
uint32 BcmMuUnlock(BcmHandle mu);
uint32 BcmMuDelete(BcmHandle mu);
uint32 BcmSemCreate(BcmHandle * sem);
uint32 BcmSemDelete(BcmHandle sem);
uint32 BcmSemLock(BcmHandle sem);
uint32 BcmSemTimedLock( BcmHandle sem, uint32 timeoutMsec );
uint32 BcmSemUnlock(BcmHandle sem);
uint32 BcmCreateEvent(uint32 tid);
int32  BcmWaitForEvent(uint32 event, uint32 * rEvent, uint32 msecs, uint32 tid);
int32  BcmSendEvent(uint32 event, uint32 tid);
int32  BcmCreateThread(char * tname, uint32 tprio, uint32 tstack, uint32 * tid);
int32  BcmStartThread(uint32 tid, uint32 (*tfunc)(void *), void * targs);
int32  BcmSuspendThread(uint32 tid);
int32  BcmResumeThread(uint32 tid);
uint32 BcmStartTimerPeriodic(int32 ticks, uint32 event, uint32 tid);
uint32 BcmStartTimerOneShot(int32 ticks, uint32 event, uint32 tid);
uint32 BcmCancelTimer(uint32 * t_handle);
uint32 TimeMs(void);
uint32 BcmMsgQCreate(char *MsgQName, uint32 MsgQSize, uint32 MaxMsgSize);
void   BcmMsgQDelete(uint32 MsgQid);
uint32 BcmMsgQSend(uint32 msgQId, uint32 *Msg, UINT16 MsgLen);
uint32 BcmMsgQReceive(uint32 msgQId, uint32 *AvailMsgBuffer, uint32 AvailMsgBufferLen, 
                                                           uint32 *RcvdMsgBufferLen,BOOL WaitFlag);

//interrupt.c
void   BcmHalInterruptDisable(uint32 interruptId);
void   BcmHalInterruptEnable(uint32 interruptId);
uint32 BcmHalMapInterrupt(uint32 (*pfunc)(), uint32 interruptId);

//int    BcmHalCpuInterruptDisable(void);
//void   BcmHalCpuInterruptEnable( uint32 );
//#define BcmHalCpuInterruptDisable() intLock()
//#define BcmHalCpuInterruptEnable(lvl) intUnlock(lvl)

// These macros are designed to look and behave like the VxWorks intLock and
// intUnlock functions (they are taken directly from those functions).  The
// difference is that they are inline, which not only eliminates the function
// call/return overhead, but also allows intelligent register use by the
// compiler.  These macros are coded for the GNU C compiler inline asm syntax,
// and for the MIPS CPU.  Since this is the VxWorks version of bcmos.h, and we
// only use the GNU compiler for VxWorks, and we only care about MIPS processors
// this should be relatively reasonable.
//
// Note the use of (), which groups the instructions into a single expression,
// the result of which is returned essentially as though it were a function call.
// In order to store and return the result, we need to introduce a new scope via
// {} so that we can have temporary variables.  This combination of ({...}) is
// not ANSI compliant, but works with the GNU compiler.  In order to keep the
// compiler from warning about this non-ANSI construct, the GNU boys allow us
// to use the __extension__ keywork, which suppresses the non-ANSI warning for
// this macro.
#define BcmHalCpuInterruptDisable()                                                         \
__extension__                                                                               \
({                                                                                          \
    /* These variables will be register-optimized by the compiler. */                       \
    uint32 __oldLevel, __temp = 0;                                                               \
                                                                                            \
    /* This sequence of instructions moves the current CP0, register 12 value */            \
    /* to __oldLevel, masks off the LSB (storing the masked value in __temp), */            \
    /* then stores the __temp result back to CP0 register 12.  The volatile   */            \
    /* keyword keeps the compiler from reordering the statements too much.    */            \
    __asm__ volatile ("mfc0 %0, $12" : "=d" (__oldLevel));                                  \
    __asm__ volatile ("addiu %0, $0, 0xfffe" : /* no outputs */ : "d" (__temp));            \
    __asm__ volatile ("and %0, %1, %2" : "=d" (__temp) : "d" (__temp), "d" (__oldLevel));   \
    __asm__ volatile ("mtc0 %0, $12" : /* no outputs */ : "d" (__temp));                    \
                                                                                            \
    /* Because there is a 2 cycle write delay/hazard on mtc0, interrupts won't */           \
    /* be disabled until 2 instructions after the mtc0.  Since the assembler/  */           \
    /* compiler doesn't know this, we have to issue 'nop's to force the delay  */           \
    /* so that the code that follows BcmHalCpuInterruptDisable() can be sure   */           \
    /* that interrupts really are disabled.                                    */           \
    __asm__ ("nop; nop");                                                                   \
                                                                                            \
    /* This causes the value in __oldLevel to be returned from the macro as */              \
    /* though it were the return value from a function.                     */              \
    __oldLevel;                                                                             \
})

#define BcmHalCpuInterruptEnable(__oldLevel)                                                \
    /* This instruction moves the value in __oldLevel to CP0 register 12. */                \
    /* The __oldLevel value should be the value that was returned from    */                \
    /* BcmHalCpuInterruptDisable().                                       */                \
    __asm__ volatile ("mtc0 %0, $12" : /* no outputs */ : "d" (__oldLevel))


#define BcmHalInterruptDone   BcmHalInterruptEnable

typedef struct
{
    uint32 lockCount;
    uint32 waitCount;

    uint32 lockOwnerTaskId;

    BcmHandle semaphore;

} BcmMutexLiteData;

typedef BcmMutexLiteData * BcmMutexLite;

uint32 BcmMuLiteCreate(BcmMutexLite *mu);
uint32 BcmMuLiteDelete(BcmMutexLite mu);

#define BcmMuLiteLock(/*BcmMutexLite*/ mu)                                              \
{                                                                                       \
    uint32 oldLevel;                                                                     \
                                                                                        \
    /* Because the 'mu' parameter is likely to be a global variable, the compiler    */ \
    /* will refuse to register optimize it (it will recalculate the address over and */ \
    /* over in this macro).  This essentially tells the compiler to register optimize*/ \
    /* the address.                                                                  */ \
    BcmMutexLiteData *pMutexData = mu;                                                  \
                                                                                        \
    /* Same kind of thing with taskIdCurrent; we can read it once and keep it local. */ \
    uint32 taskId = (uint32) cyg_thread_self();                                         \
                                                                                        \
    oldLevel = BcmHalCpuInterruptDisable();                                             \
                                                                                        \
    /* If somebody currently has the lock, and it isn't me, then I need to */           \
    /* wait for the lock.                                                  */           \
    if ((pMutexData->lockCount > 0) && (pMutexData->lockOwnerTaskId != taskId))         \
    {                                                                                   \
        /* Note that I use a while loop rather than an if statement; this is because */ \
        /* there is a possible race condition during the unlock where we can         */ \
        /* decrement the count to 0 before unlocking the semaphore; if the unlocking */ \
        /* task is interrupted then the locking task can blow through this while loop*/ \
        /* and get the lock.  Thus, when task is waiting on the semaphore, it needs  */ \
        /* to check the lock state again after waking up.                            */ \
        do                                                                              \
        {                                                                               \
            /* Increment the count of tasks waiting for the lock, then restore */       \
            /* interrupts. */                                                           \
            pMutexData->waitCount++;                                                    \
            BcmHalCpuInterruptEnable(oldLevel);                                         \
                                                                                        \
            /* Now go to sleep, yielding the processor until somebody unlocks the */    \
            /* mutex and releases the counting semaphore. */                            \
            BcmSemLock(pMutexData->semaphore);                                          \
                                                                                        \
            /* Lock out interrupts and decrement the count of tasks waiting. */         \
            oldLevel = BcmHalCpuInterruptDisable();                                     \
            pMutexData->waitCount--;                                                    \
        }                                                                               \
        while (pMutexData->lockCount > 0);                                              \
    }                                                                                   \
                                                                                        \
    /* PR6583 - Else, if someone else is waiting for the semaphore (this only   */      \
    /* happens if the other thread is a lower priority than me), then let them  */      \
    /* run and get the lock.                                                    */      \
    else if (pMutexData->waitCount > 0)                                                 \
    {                                                                                   \
        BcmHalCpuInterruptEnable(oldLevel);                                             \
                                                                                        \
        Sleep(1);                                                                       \
                                                                                        \
        oldLevel = BcmHalCpuInterruptDisable();                                         \
    }                                                                                   \
                                                                                        \
    /* Increment the lock count and set myself as the lock owner. */                    \
    pMutexData->lockCount++;                                                            \
    pMutexData->lockOwnerTaskId = taskId;                                               \
                                                                                        \
    BcmHalCpuInterruptEnable(oldLevel);                                                 \
}


#define BcmMuLiteUnlock(/*BcmMutexLite*/ mu)                                            \
{                                                                                       \
    /* Because the 'mu' parameter is likely to be a global variable, the compiler    */ \
    /* will refuse to register optimize it (it will recalculate the address over and */ \
    /* over in this macro).  This essentially tells the compiler to register optimize*/ \
    /* the address. */                                                                  \
    BcmMutexLiteData *pMutexData = mu;                                                  \
                                                                                        \
    /* Note that I don't need to disable interrupts around this.  If I am interrupted*/ \
    /* everything will work out ok, since either the thread will go into the wait    */ \
    /* loop, and I'll wake them up, or it will get the lock and I won't need to wake */ \
    /* anybody up.                                                                   */ \
    pMutexData->lockCount--;                                                            \
                                                                                        \
    /* PR7430 - only unlock the semaphore if this is the last lock that I have on    */ \
    /* the mutex.  We support recursive locks, and I don't want to give someone else */ \
    /* access to the mutex until I'm completely finished with it.                    */ \
    if ((pMutexData->lockCount == 0) && (pMutexData->waitCount > 0))                    \
    {                                                                                   \
        BcmSemUnlock(pMutexData->semaphore);                                            \
                                                                                        \
        /* PR6583 - How lame is this - the eCos counting semaphore implementation    */ \
        /* doesn't work correctly.  If a thread is waiting for the semaphore (even a */ \
        /* higher priority thread!), it doesn't get a chance to run.  As a result, I */ \
        /* have to get the current thread to sleep a bit which should let the slacker*/ \
        /* OS give the waiting thread a chance to go (if it is equal or higher       */ \
        /* priority).  Note that this same problem exists in VxWorks!                */ \
        Sleep(0);                                                                       \
    }                                                                                   \
}


// DPullen - replaced the duplicate prototypes of the BcmAlloc API with an
// include of bcmAlloc.h, which is the only place where the API should be.
#include "bcmAlloc.h"

BOOL HalMapInterrupt(IN uint32 InterruptNumber,IN uint32 InterruptVector, OUT DWORD *InterruptId);


//**********************Special Section*********************************
// This section is dedicated to pSOS defines that are used in the code
// but cannot be found in VxWorks 
//**********************************************************************
//redefine C preprocessor for packed, Vxworks support diff syntax
#define __packed__
#define GNU_PACKED		__attribute__ ((packed))

#ifndef	htonl
/*---------------------------------------------------------------------*/
/* Definitions for Network Order Conversions Routines                  */
/*---------------------------------------------------------------------*/
#if BSP_LITTLE_ENDIAN
    /*-----------------------------------------------------------------*/
    /* Change byte order so the least significant byte is stored at    */
    /* the lowest address. Unsigned to prevent integer overflow on     */
    /* i960. Big endian cast unsigned for constant handling of vars.   */
    /*-----------------------------------------------------------------*/
    #define htonl(addr)     ( (((unsigned long)(addr) & 0x000000FF)<<24) | \
                             (((unsigned long)(addr) & 0x0000FF00)<<8)  | \
                             (((unsigned long)(addr) & 0x00FF0000)>>8)  | \
                             (((unsigned long)(addr) & 0xFF000000)>>24))

    #define ntohl(addr)     htonl(addr)

    #define htons(addr)     ( (((unsigned short)(addr) & 0x000000FF)<<8)  | \
                             (((unsigned short)(addr) & 0x0000FF00)>>8))

    #define ntohs(addr)     htons(addr)
#endif
#endif


#define ERR_TIMEOUT  0x01     /* Timed out; returned only if timeout   */
                              /* was requested                         */


//#ifndef	_ASMLANGUAGE

typedef enum parm_type
    {
    FLAG = 1, IP = 2, HEX = 3, DECIMAL = 4, CHAR = 5, STRING = 6
    } PARM_TYPE;

/*---------------------------------------------------------------------*/
/* These parameters correspond to the SD_ parameters in sys_conf.h     */
/* They are the parameters which may be stored and retrieved in the    */
/* board's parameter storage area.  In addition, the optional startup  */
/* configuration dialog can be used to change them.                    */
/*---------------------------------------------------------------------*/
struct SD_parms
    {
    /*-----------------------------------------------------------------*/
    /* Debug Mode determines how pROBE+ wil operate:                   */
    /*   DBG_SA - Standalone mode                                      */
    /*   DBG_XS - Communicating with host debugger via serial link     */
    /*   DBG_XN - Communicating with host debugger via network         */
    /*   DBG_AP - Start the application immediately (using "silent     */
    /*            startup").  pROBE+ should be configured to run in    */
    /*            standalone mode.                                     */
    /*-----------------------------------------------------------------*/
    unsigned char DebugMode;

    /*-----------------------------------------------------------------*/
    /* RbugDrvType determines how pROBE+ will connect to host debugger.*/
    /* if the Debug Mode is DBG_AP or DBG_SA and more than one type    */
    /* of the drivers available, for exmaple, both serial or network   */
    /* connection are available for connecting to the host debugger.   */
    /*  RBUG_DRV_SERIAL: use the serial port if possible               */
    /*  RBUG_DRV_NET:    use the network connection if possible        */
    /*-----------------------------------------------------------------*/
    unsigned char RbugDrvType;

    unsigned char Pad1[2];

    /*-----------------------------------------------------------------*/
    /* These variables control miscellaneous basic characteristics     */
    /* of the system:                                                  */
    /*   StartupDelay - If the startup configuration dialog is         */
    /*                  configured, it will wait this many seconds     */
    /*                  for input from the serial channel              */
    /*   DefBaud - Initial baud rate to use for the serial channels.   */
    /*             (Applications may change the baud rate via          */
    /*             de_ctrl() calls)                                    */
    /*   VmeBaseAddr - The base address of the board's dual-ported     */
    /*                 memory, as seen from the VMEbus.  Not used      */
    /*                 on non-VME boards (BSP_VME = NO)                */
    /*-----------------------------------------------------------------*/
    unsigned long StartupDelay;
    unsigned long DefBaud;
    unsigned long VmeBaseAddr;

    /*-----------------------------------------------------------------*/
    /* DefGtwyIP is used when networking is enabled to specify a       */
    /* "default gateway" node where packets should be sent in the      */
    /* absence of other routing information.                           */
    /*-----------------------------------------------------------------*/
    unsigned long DefGtwyIP;

    /*-----------------------------------------------------------------*/
    /* These variables control the LAN interface.  If SC_PNA and/or    */
    /* BSP_LAN1 is FALSE, they are ignored.                            */
    /*   Lan1 - TRUE if LAN interface should be enabled                */
    /*   Lan1IP - IP address for the LAN interface.  0 indicates that  */
    /*            RARP should be used to obtain the IP address.        */
    /*   Lan1SubnetMask - Subnet mask to use for the LAN interface     */
    /*-----------------------------------------------------------------*/
    unsigned char Lan1;
    unsigned char Pad2[3];
    unsigned long Lan1IP;
    unsigned long Lan1SubnetMask;

    /*-----------------------------------------------------------------*/
    /* If a shared memory interface is to be used (either NI or KI or  */
    /* both), it must have a node number, which is a small integer.    */
    /* SmNode holds this number.                                       */
    /*-----------------------------------------------------------------*/
    unsigned long SmNode;  

    /*-----------------------------------------------------------------*/
    /* These variables control the shared memory network interface.    */
    /*   Nism - TRUE if SMNI should be enabled                         */
    /*   NismIP - IP address to use for the interface                  */
    /*   NismSubnetMask - Subnet mask to use for the interface         */
    /*   NismDiraddr - Bus address of the system-wide "directory"      */
    /*           structure for the interface.                          */
    /* Note that SmNode, above, is also relevant to the SMNI.          */
    /*-----------------------------------------------------------------*/
    unsigned char Nism;   
    unsigned char Pad3[3];
    unsigned long NismIP;  
    unsigned long NismSubnetMask;
    unsigned long NismDiraddr;

    /*-----------------------------------------------------------------*/
    /* These variables control the shared memory kernel interface.     */
    /*   Kism - 0 if KISM should not be enabled.  A non-zero value     */
    /*          indicates that the SMKI should be enabled.  In this    */
    /*          case "Kism" should be set to the total number of nodes */
    /*          in the system.  SmNode (defined above here) is set     */
    /*          to this node's node number, which must be in the range */
    /*          1 through "Kism".                                      */
    /*   KismDiraddr - Bus address of the system-wide "directory"      */
    /*          structure for the interface                            */
    /*-----------------------------------------------------------------*/
    unsigned long Kism;
    unsigned long KismDiraddr;
    }; 

typedef struct NV_Parms {
    unsigned long magic;
    unsigned long SysVars_CheckSum;
    struct SD_parms SysVars;
 
#if BSP_PARMS
    unsigned long BspVars_CheckSum;
    uint8 BspVars[BSP_PARMS];
#endif
 
#if SC_APP_PARMS
    unsigned long AppVars_CheckSum;
    uint8 AppVars[SC_APP_PARMS];
#endif
} NV_Parms;

extern NV_Parms NVVars;

#define SysVars NVVars.SysVars
#define BspVars NVVars.BspVars
#define AppVars NVVars.AppVars


/*---------------------------------------------------------------------*/
/* mu_create() Definitions                                             */
/*---------------------------------------------------------------------*/
#define MU_GLOBAL       0x00000001  /* 1 = Global */
#define MU_LOCAL        0x00000000  /* 0 = Local */
#define MU_PRIOR        0x00000002  /* Queue by priority */
#define MU_FIFO         0x00000000  /* Queue by FIFO order */
#define MU_RECURSIVE    0x00000004  /* Recursive acquisition Allowed */
#define MU_NORECURSIVE  0x00000000  /* Recursive acquisition Not Allowed */
#define MU_PRIO_NONE    0x00000000  /* No unbounded priority inversion */
                                    /* protection */
#define MU_PRIO_INHERIT 0x00000028  /* Priority Inheritance protocol */
                                    /* enabled */
#define MU_PRIO_PROTECT 0x00000008  /* Priority Protection protocol */
                                    /* enabled */
/*---------------------------------------------------------------------*/
/* mu_lock() Definitions                                               */
/*---------------------------------------------------------------------*/
#define MU_NOWAIT       0x00000001  /* Don't wait for lock */
#define MU_WAIT         0x00000000  /* Wait for lock */
 
//#endif



//---------------------------- extracted from pSOS --------------------------
//The following defines are required for building the exisiting pSOS code 
//under VxWorks

//extracted from psos.h
/*---------------------------------------------------------------------*/
/* sm_p() Definitions                                                  */
/*---------------------------------------------------------------------*/
#define SM_WAIT         0x00000000  /* Wait for semaphore */
/*---------------------------------------------------------------------*/
/* ev_receive() Definitions                                            */
/*---------------------------------------------------------------------*/
#define EV_WAIT         0x00000000  /* Wait for events */
/*---------------------------------------------------------------------*/
/* as_catch(), t_mode(), and t_start() Definitions                     */
/*---------------------------------------------------------------------*/
#define T_ISR           0x00000000   /* Interrupts enabled */

//hpna required start
/*---------------------------------------------------------------------*/
/* Structure of pna functions passed to NI during ni_init              */
/*---------------------------------------------------------------------*/
struct free_rtn
    {
    void   (*free_func)(void *);
    void   *free_arg;
    };
typedef struct free_rtn frtn_t;

struct msgb
    {
    struct msgb    *b_next;    /* next msg on queue */
    struct msgb    *b_prev;    /* previous msg on queue */
    struct msgb    *b_cont;    /* next msg block of msg */
    unsigned char  *b_rptr;    /* first unread data byte in buffer */
    unsigned char  *b_wptr;    /* first unwritten data byte in buffer */
    struct datab   *b_datap;   /* data block */
    short           whichp;    /* used internally */
    short           reserved;  /* Reserved for future use */
    };
typedef struct msgb mblk_t;

struct ni_funcs
    {
    mblk_t *(*allocb)(long, long);  /* Allocate pNA buffer */
    void   (*freeb)(mblk_t *);      /* Free Message block */
    void   (*freemsg)(mblk_t *);    /* Free message */
//    mblk_t *(*esballoc)(uint8 *, int, int, frtn_t *);
                                    /* Attach message block */
    void   (*intf_cb)(int, char*, int, long);
                                    /* pNA callback for interface operations */
    };

//hpna required end

//---------------------------- extracted from pSOS --------------------------

//***************Special section end ************************************

//extern unsigned int clock_frequency;

#define	K0_TO_K1(x)	((unsigned)(x)|0xA0000000)	/* kseg0 to kseg1 */
#define	K1_TO_K0(x)	((unsigned)(x)&0x9FFFFFFF)	/* kseg1 to kseg0 */

#define	K0_TO_PHYS(x)	((unsigned)(x)&0x1FFFFFFF)	/* kseg0 to physical */
#define	K1_TO_PHYS(x)	((unsigned)(x)&0x1FFFFFFF)	/* kseg1 to physical */

#define	PHYS_TO_K0(x)	((unsigned)(x)|0x80000000)	/* physical to kseg0 */
#define	PHYS_TO_K1(x)	((unsigned)(x)|0xA0000000)	/* physical to kseg1 */

extern uint32 intCRGet(void);
extern uint32 intSRGet(void);


#if __cplusplus
}
#endif

#endif

